#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2012-2016 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import omero
import time
import re
import logging
import traceback
from future.utils import isbytes, bytes_to_native_str

from omero.rtypes import unwrap
from omero_marshal import get_encoder

logger = logging.getLogger(__name__)

# OMERO.insight point list regular expression
INSIGHT_POINT_LIST_RE = re.compile(r'points\[([^\]]+)\]')

# OME model point list regular expression
OME_MODEL_POINT_LIST_RE = re.compile(r'([\d.]+),([\d.]+)')


def eventContextMarshal(event_context):
    """
    Marshals the omero::sys::EventContext as a dict.

    @param event_context:   omero::sys::EventContext
    @return:                Dict
    """

    ctx = {}
    for a in ['shareId', 'sessionId', 'sessionUuid', 'userId', 'userName',
              'sudoerId', 'sudoerName', 'groupId',
              'groupName', 'isAdmin', 'eventId', 'eventType',
              'memberOfGroups', 'leaderOfGroups',
              'adminPrivileges']:
        if (hasattr(event_context, a)):
            ctx[a] = unwrap(getattr(event_context, a))

    perms = event_context.groupPermissions
    encoder = get_encoder(perms.__class__)
    ctx['groupPermissions'] = encoder.encode(perms)

    return ctx


def channelMarshal(channel):
    """
    return a dict with all there is to know about a channel

    @param channel:     L{omero.gateway.ChannelWrapper}
    @return:            Dict
    """

    chan = {'emissionWave': channel.getEmissionWave(),
            'label': channel.getLabel(),
            'color': channel.getColor().getHtml(),
            # 'reverseIntensity' is deprecated. Use 'inverted'
            'inverted': channel.isInverted(),
            'reverseIntensity': channel.isInverted(),
            'family': unwrap(channel.getFamily()),
            'coefficient': unwrap(channel.getCoefficient()),
            'window': {'min': channel.getWindowMin(),
                       'max': channel.getWindowMax(),
                       'start': channel.getWindowStart(),
                       'end': channel.getWindowEnd()},
            'active': channel.isActive()}
    lut = channel.getLut()
    if lut and len(lut) > 0:
        chan['lut'] = lut
    return chan


def imageMarshal(image, key=None, request=None):
    """
    return a dict with pretty much everything we know and care about an image,
    all wrapped in a pretty structure.

    @param image:   L{omero.gateway.ImageWrapper}
    @param key:     key of specific attributes to select
    @return:        Dict
    """

    image.loadRenderOptions()
    pr = image.getProject()
    ds = None
    wellsample = None
    well = None
    try:
        # Replicating the functionality of the deprecated
        # ImageWrapper.getDataset() with shares in mind.
        # -- Tue Sep  6 10:48:47 BST 2011 (See #6660)
        parents = image.listParents()
        if parents is not None:
            datasets = [p for p in parents if p.OMERO_CLASS == 'Dataset']
            well_smpls = [p for p in parents if p.OMERO_CLASS == 'WellSample']
            if len(datasets) == 1:
                ds = datasets[0]
            if len(well_smpls) == 1:
                if well_smpls[0].well is not None:
                    well = well_smpls[0].well
    except omero.SecurityViolation as e:
        # We're in a share so the Image's parent Dataset cannot be loaded
        # or some other permissions related issue has tripped us up.
        logger.warn('Security violation while retrieving Dataset when '
                    'marshaling image metadata: %s' % e.message)

    rv = {
        'id': image.id,
        'meta': {
            'imageName': image.name or '',
            'imageDescription': image.description or '',
            'imageAuthor': image.getAuthor(),
            'projectName': pr and pr.name or 'Multiple',
            'projectId': pr and pr.id or None,
            'projectDescription': pr and pr.description or '',
            'datasetName': ds and ds.name or 'Multiple',
            'datasetId': ds and ds.id or None,
            'datasetDescription': ds and ds.description or '',
            'wellSampleId': wellsample and wellsample.id or '',
            'wellId': well and well.id.val or '',
            'imageTimestamp': time.mktime(
                image.getDate().timetuple()),
            'imageId': image.id,
            'pixelsType': image.getPixelsType(),
            },
        'perms': {
            'canAnnotate': image.canAnnotate(),
            'canEdit': image.canEdit(),
            'canDelete': image.canDelete(),
            'canLink': image.canLink()
            }
        }
    try:
        reOK = image._prepareRenderingEngine()
        if not reOK:
            logger.debug(
                "Failed to prepare Rendering Engine for imageMarshal")
            return rv
    except omero.ConcurrencyException as ce:
        backOff = ce.backOff
        rv = {
            'ConcurrencyException': {
                'backOff': backOff
            }
        }
        return rv
    except Exception as ex:   # Handle everything else.
        rv['Exception'] = ex.message
        logger.error(traceback.format_exc())
        return rv       # Return what we have already, in case it's useful

    # big images
    tiles = image._re.requiresPixelsPyramid()
    rv['tiles'] = tiles
    if (tiles):
        width, height = image._re.getTileSize()
        levels = image._re.getResolutionLevels()
        zoomLevelScaling = image.getZoomLevelScaling()

        rv.update({'tile_size': {'width': width,
                                 'height': height},
                   'levels': levels})
        if zoomLevelScaling is not None:
            rv['zoomLevelScaling'] = zoomLevelScaling

    nominalMagnification = (
        image.getObjectiveSettings() is not None and
        image.getObjectiveSettings().getObjective().getNominalMagnification()
        or None)

    try:
        server_settings = request.session.get('server_settings', {}) \
                                         .get('viewer', {})
    except Exception:
        server_settings = {}
    init_zoom = server_settings.get('initial_zoom_level', 0)
    if init_zoom < 0:
        init_zoom = levels + init_zoom

    interpolate = server_settings.get('interpolate_pixels', True)

    try:
        def pixel_size_in_microns(method):
            try:
                size = method('MICROMETER')
                return size.getValue() if size else None
            except Exception:
                logger.debug(
                    'Unable to convert physical pixel size to microns',
                    exc_info=True
                )
                return None

        rv.update({
            'interpolate': interpolate,
            'size': {'width': image.getSizeX(),
                     'height': image.getSizeY(),
                     'z': image.getSizeZ(),
                     't': image.getSizeT(),
                     'c': image.getSizeC()},
            'pixel_size': {'x': pixel_size_in_microns(image.getPixelSizeX),
                           'y': pixel_size_in_microns(image.getPixelSizeY),
                           'z': pixel_size_in_microns(image.getPixelSizeZ)},
            })
        if init_zoom is not None:
            rv['init_zoom'] = init_zoom
        if nominalMagnification is not None:
            rv.update({'nominalMagnification': nominalMagnification})
        try:
            rv['pixel_range'] = image.getPixelRange()
            rv['channels'] = [channelMarshal(x) for x in image.getChannels()]
            rv['split_channel'] = image.splitChannelDims()
            rv['rdefs'] = {'model': (image.isGreyscaleRenderingModel() and
                                     'greyscale' or 'color'),
                           'projection': image.getProjection(),
                           'defaultZ': image._re.getDefaultZ(),
                           'defaultT': image._re.getDefaultT(),
                           'invertAxis': image.isInvertedAxis()}
        except TypeError:
            # Will happen if an image has bad or missing pixel data
            logger.error('imageMarshal', exc_info=True)
            rv['pixel_range'] = (0, 0)
            rv['channels'] = ()
            rv['split_channel'] = ()
            rv['rdefs'] = {'model': 'color',
                           'projection': image.getProjection(),
                           'defaultZ': 0,
                           'defaultT': 0,
                           'invertAxis': image.isInvertedAxis()}
    except AttributeError:
        rv = None
        raise
    if key is not None and rv is not None:
        for k in key.split('.'):
            rv = rv.get(k, {})
        if rv == {}:
            rv = None
    return rv


def shapeMarshal(shape):
    """
    return a dict with all there is to know about a shape

    @param channel:     L{omero.model.ShapeI}
    @return:            Dict
    """
    rv = {}

    def set_if(k, v, func=lambda a: a is not None):
        """
        Sets the key L{k} with the value of L{v} if the unwrapped value L{v}
        passed to L{func} evaluates to True.  In the default case this is
        True if the unwrapped value L{v} is not None.
        """
        v = unwrap(v)
        if func(v):
            rv[k] = v

    def decode(shape_field):
        value = shape_field.getValue()
        if value and isbytes(value):
            value = bytes_to_native_str(value)
        return value

    rv['id'] = shape.getId().getValue()
    set_if('theT', shape.getTheT())
    set_if('theZ', shape.getTheZ())
    shape_type = type(shape)
    if shape_type == omero.model.RectangleI:
        rv['type'] = 'Rectangle'
        rv['x'] = shape.getX().getValue()
        rv['y'] = shape.getY().getValue()
        rv['width'] = shape.getWidth().getValue()
        rv['height'] = shape.getHeight().getValue()
    elif shape_type == omero.model.MaskI:
        rv['type'] = 'Mask'
        rv['x'] = shape.getX().getValue()
        rv['y'] = shape.getY().getValue()
        rv['width'] = shape.getWidth().getValue()
        rv['height'] = shape.getHeight().getValue()
        # TODO: support for mask
    elif shape_type == omero.model.EllipseI:
        rv['type'] = 'Ellipse'
        rv['x'] = shape.getX().getValue()
        rv['y'] = shape.getY().getValue()
        rv['radiusX'] = shape.getRadiusX().getValue()
        rv['radiusY'] = shape.getRadiusY().getValue()
    elif shape_type == omero.model.PolylineI:
        rv['type'] = 'PolyLine'
        points = decode(shape.getPoints())
        rv['points'] = stringToSvg(points)
    elif shape_type == omero.model.LineI:
        rv['type'] = 'Line'
        rv['x1'] = shape.getX1().getValue()
        rv['x2'] = shape.getX2().getValue()
        rv['y1'] = shape.getY1().getValue()
        rv['y2'] = shape.getY2().getValue()
    elif shape_type == omero.model.PointI:
        rv['type'] = 'Point'
        rv['x'] = shape.getX().getValue()
        rv['y'] = shape.getY().getValue()
    elif shape_type == omero.model.PolygonI:
        rv['type'] = 'Polygon'
        # z = closed line
        points = decode(shape.getPoints())
        rv['points'] = stringToSvg(points) + " z"
    elif shape_type == omero.model.LabelI:
        rv['type'] = 'Label'
        rv['x'] = shape.getX().getValue()
        rv['y'] = shape.getY().getValue()
    else:
        logger.debug("Shape type not supported: %s" % str(shape_type))

    text_value = unwrap(shape.getTextValue())
    if text_value is not None:
        # only populate json with font styles if we have some text
        rv['textValue'] = text_value
        # FIXME: units ignored for font size
        if shape.getFontSize() is not None:
            set_if('fontSize', shape.getFontSize().getValue())
        set_if('fontStyle', shape.getFontStyle())
        set_if('fontFamily', shape.getFontFamily())

    if shape.getTransform() is not None:
        transform = shape.getTransform()
        tm = [unwrap(transform.a00),
              unwrap(transform.a10),
              unwrap(transform.a01),
              unwrap(transform.a11),
              unwrap(transform.a02),
              unwrap(transform.a12)]
        rv['transform'] = 'matrix(%s)' % (' '.join([str(t) for t in tm]))
    fill_color = unwrap(shape.getFillColor())
    if fill_color is not None:
        rv['fillColor'], rv['fillAlpha'] = rgb_int2css(fill_color)
    stroke_color = unwrap(shape.getStrokeColor())
    if stroke_color is not None:
        rv['strokeColor'], rv['strokeAlpha'] = rgb_int2css(stroke_color)
    if shape.getStrokeWidth() is not None:
        # FIXME: units ignored for stroke width
        set_if('strokeWidth', shape.getStrokeWidth().getValue())
    if hasattr(shape, 'getMarkerStart') and shape.getMarkerStart() is not None:
        # Handle string for python2 and bytes python3. TODO: lower level fix
        rv['markerStart'] = decode(shape.getMarkerStart())
    if hasattr(shape, 'getMarkerEnd') and shape.getMarkerEnd() is not None:
        rv['markerEnd'] = decode(shape.getMarkerEnd())
    return rv


def stringToSvg(string):
    """
    Method for converting the string returned from
    omero.model.ShapeI.getPoints() into an SVG for display on web.
    E.g: "points[309,427, 366,503, 190,491] points1[309,427, 366,503, 190,491]
          points2[309,427, 366,503, 190,491]"
    To: M 309 427 L 366 503 L 190 491 z
    """
    point_list = string.strip()
    match = INSIGHT_POINT_LIST_RE.search(point_list)
    if match is not None:
        point_list = match.group(1)
    point_list = OME_MODEL_POINT_LIST_RE.findall(point_list)
    if len(point_list) == 0:
        logger.error("Unrecognised ROI shape 'points' string: %r" % string)
        return ""
    point_list = ' L '.join([' '.join(point) for point in point_list])
    return "M %s" % point_list


def rgb_int2css(rgbint):
    """
    converts a bin int number into css colour and alpha fraction.
    E.g. -1006567680 to '#00ff00', 0.5
    """
    alpha = rgbint % 256
    alpha = float(alpha) / 255
    b = rgbint // 256 % 256
    g = rgbint // 256 // 256 % 256
    r = rgbint // 256 // 256 // 256 % 256
    return "#%02x%02x%02x" % (r, g, b), alpha


def rgb_int2rgba(rgbint):
    """
    converts a bin int number into (r, g, b, alpha) tuple.
    E.g. 1694433280 to (255, 0, 0, 0.390625)
    """
    alpha = rgbint % 256
    alpha = float(alpha) / 255
    b = rgbint // 256 % 256
    g = rgbint // 256 // 256 % 256
    r = rgbint // 256 // 256 // 256 % 256
    return (r, g, b, alpha)


def chgrpMarshal(conn, rsp):
    """
    Helper for marshalling a Chgrp response.
    Uses conn to lookup unlinked objects.
    Returns dict of e.g.
    {'includedObjects': {'Datasets':[1,2,3]},
     'unlinkedDetails': {'Tags':[{'id':1, 'name':'t'}]}
     }
    """
    rv = {}
    if isinstance(rsp, omero.cmd.ERR):
        rsp_params = ", ".join(["%s: %s" % (k, v) for k, v in
                               rsp.parameters.items()])
        rv['error'] = rsp.message
        rv['report'] = "%s %s" % (rsp.name, rsp_params)
    else:
        included = rsp.responses[0].includedObjects
        deleted = rsp.responses[0].deletedObjects

        # Included: just simplify the key, e.g. -> Projects, Datasets etc
        includedObjects = {}
        objKeys = ['ome.model.containers.Project',
                   'ome.model.containers.Dataset',
                   'ome.model.core.Image',
                   'ome.model.screen.Screen',
                   'ome.model.screen.Plate',
                   'ome.model.screen.Well']
        for k in objKeys:
            if k in included:
                otype = k.split(".")[-1]
                oids = included[k]
                oids.sort()     # makes testing easier
                includedObjects[otype + 's'] = oids
        rv['includedObjects'] = includedObjects

        # Annotation links - need to get info on linked objects
        tags = {}
        files = {}
        comments = 0
        others = 0
        annotationLinks = ['ome.model.annotations.ProjectAnnotationLink',
                           'ome.model.annotations.DatasetAnnotationLink',
                           'ome.model.annotations.ImageAnnotationLink',
                           'ome.model.annotations.ScreenAnnotationLink',
                           'ome.model.annotations.PlateAnnotationLink',
                           'ome.model.annotations.WellAnnotationLink']
        for l in annotationLinks:
            if l in deleted:
                linkType = l.split(".")[-1]
                params = omero.sys.ParametersI()
                params.addIds(deleted[l])
                query = ("select annLink from %s as annLink "
                         "join fetch annLink.child as ann "
                         "left outer join fetch ann.file "
                         "where annLink.id in (:ids)" % linkType)
                links = conn.getQueryService().findAllByQuery(
                    query, params, conn.SERVICE_OPTS)
                for lnk in links:
                    ann = lnk.child
                    if isinstance(ann, omero.model.FileAnnotationI):
                        name = unwrap(ann.getFile().getName())
                        files[ann.id.val] = {'id': ann.id.val,
                                             'name': name}
                    elif isinstance(ann, omero.model.TagAnnotationI):
                        name = unwrap(ann.getTextValue())
                        tags[ann.id.val] = {'id': ann.id.val,
                                            'name': name}
                    elif isinstance(ann, omero.model.CommentAnnotationI):
                        comments += 1
                    else:
                        others += 1
        # sort tags & comments
        tags = list(tags.values())
        tags.sort(key=lambda x: x['name'])
        files = list(files.values())
        files.sort(key=lambda x: x['name'])
        rv['unlinkedDetails'] = {'Tags': tags,
                                 'Files': files,
                                 'Comments': comments,
                                 'Others': others
                                 }

        # Container links - only report these if we are moving the *parent*,
        # E.g. DatasetImageLinks are only reported if we are moving the Dataset
        # (and the image is left behind). If we were moving the Image then we'd
        # expect the link to be broken (can ignore)
        objects = {}
        containerLinks = {
            'ome.model.containers.ProjectDatasetLink': 'Datasets',
            'ome.model.containers.DatasetImageLink': 'Images',
            'ome.model.screen.ScreenPlateLink': 'Screens'}
        for l, ch in containerLinks.items():
            if l in deleted:
                linkType = l.split(".")[-1]
                params = omero.sys.ParametersI()
                params.addIds(deleted[l])
                query = ("select conLink from %s as conLink "
                         "join fetch conLink.child as ann "
                         "where conLink.id in (:ids)" % linkType)
                links = conn.getQueryService().findAllByQuery(
                    query, params, conn.SERVICE_OPTS)
                for lnk in links:
                    child = lnk.child
                    if (ch not in includedObjects or
                            child.id.val not in includedObjects[ch]):
                        name = unwrap(child.getName())
                        # Put objects in a dictionary to avoid duplicates
                        if ch not in objects:
                            objects[ch] = {}
                        # E.g. objects['Dataset']['1'] = {}
                        objects[ch][child.id.val] = {'id': child.id.val,
                                                     'name': name}
        # sort objects
        for otype, objs in objects.items():
            objs = list(objs.values())
            objs.sort(key=lambda x: x['name'])
            # E.g. 'Dataset' objects in 'Datasets'
            rv['unlinkedDetails'][otype] = objs

    return rv



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
#
# Copyright (c) 2008-2014 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2008.
#
# Version: 1.0
#

import os
import sys
import logging
from wsgiref.simple_server import WSGIServer

logger = logging.getLogger(__name__)

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "omeroweb.settings")

    try:
        from django.core.management import execute_from_command_line
    except ImportError:
        # The above import may fail for some other reason. Ensure that the
        # issue is really that Django is missing to avoid masking other
        # exceptions on Python 2.
        try:
            import django
        except ImportError:
            raise ImportError(
                "Couldn't import Django. Are you sure it's installed and "
                "available on your PYTHONPATH environment variable? Did you "
                "forget to activate a virtual environment?"
            )
        raise

    import settings
    from omero.util import configure_logging
    if settings.DEBUG:
        configure_logging(
            settings.LOGDIR, 'OMEROweb.log', loglevel=logging.DEBUG)

    logger.info("Application Starting...")

    # Monkeypatch Django development web server to always run in single thread
    # even if --nothreading is not specified on command line
    def force_nothreading(addr, port, wsgi_handler, ipv6=False,
                          threading=False, server_cls=WSGIServer):
        django_core_servers_basehttp_run(addr, port, wsgi_handler, ipv6,
                                         False, server_cls)
    import django.core.servers.basehttp
    if django.core.servers.basehttp.run.__module__ != 'settings':
        django_core_servers_basehttp_run = django.core.servers.basehttp.run
        django.core.servers.basehttp.run = force_nothreading

    execute_from_command_line(sys.argv)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011-2016 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.http import HttpResponse, HttpResponseServerError


class HttpJavascriptResponse(HttpResponse):
    def __init__(self, content):
        HttpResponse.__init__(self, content,
                              content_type="application/javascript")


class HttpJavascriptResponseServerError(HttpResponseServerError):
    def __init__(self, content):
        HttpResponseServerError.__init__(
            self, content, content_type="text/javascript")


class HttpJPEGResponse(HttpResponse):
    def __init__(self, content):
        HttpResponse.__init__(self, content, content_type="image/jpeg")



# -*- coding: utf-8 -*-

# Copyright (C) 2015 Glencoe Software, Inc.
# All rights reserved.
#
# Use is subject to license terms supplied in LICENSE.txt

"""
   Module to encapsulate operations concerned with displaying the contents of a
   plate as a grid.
"""

import logging

import omero.sys
from omero.rtypes import rint
try:
    import long
except ImportError:
    long = int

logger = logging.getLogger(__name__)


class PlateGrid(object):
    """
    A PlateGrid object encapsulates a PlateI reference and provides a number of
    methods useful for displaying the contents of the plate as a grid.
    """

    def __init__(self, conn, pid, fid, thumbprefix=''):
        self.plate = conn.getObject('plate', long(pid))
        self._conn = conn
        self.field = fid
        self._thumbprefix = thumbprefix
        self._metadata = None

    @property
    def metadata(self):
        if self._metadata is None:
            self.plate.setGridSizeConstraints(8, 12)
            size = self.plate.getGridSize()
            grid = [[None] * size['columns'] for _ in range(size['rows'])]

            q = self._conn.getQueryService()
            params = omero.sys.ParametersI()
            params.addId(self.plate.id)
            params.add('wsidx', rint(self.field))
            query = "select well.row, well.column, img.id, img.name, "\
                    "author.firstName||' '||author.lastName, "\
                    "well.id, img.acquisitionDate, "\
                    "img.details.creationEvent.time, "\
                    "img.description "\
                    "from Well well "\
                    "join well.wellSamples ws "\
                    "join ws.image img "\
                    "join img.details.owner author "\
                    "where well.plate.id = :id "\
                    "and index(ws) = :wsidx"

            for res in q.projection(query, params, self._conn.SERVICE_OPTS):
                row, col, img_id, img_name, author, well_id, acq_date, \
                    create_date, description = res

                if acq_date is not None and acq_date.val > 0:
                    date = acq_date.val // 1000
                else:
                    date = create_date.val // 1000
                description = (description and description.val) or ''

                wellmeta = {'type': 'Image',
                            'id': img_id.val,
                            'name': img_name.val,
                            'description': description,
                            'author': author.val,
                            'date': date,
                            'wellId': well_id.val,
                            'field': self.field}

                if callable(self._thumbprefix):
                    wellmeta['thumb_url'] = self._thumbprefix(str(img_id.val))
                else:
                    wellmeta['thumb_url'] = self._thumbprefix + str(img_id.val)

                grid[row.val][col.val] = wellmeta

            self._metadata = {'grid': grid,
                              'collabels': self.plate.getColumnLabels(),
                              'rowlabels': self.plate.getRowLabels()}
        return self._metadata



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2016 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Exceptions used by the api/views methods."""


class ApiException(Exception):
    """A base exception class that handles message and stactrace."""

    def __init__(self, message, stacktrace=None):
        """Override init to handle message and stacktrace."""
        super(ApiException, self).__init__(message)
        self.stacktrace = stacktrace


class BadRequestError(ApiException):
    """
    An exception that will result in a response status of 400.

    Due to invalid client input
    """

    status = 400


class NotFoundError(ApiException):
    """
    An exception that will result in a response status of 404.

    Raised due to objects not being found.
    """

    status = 404


class MethodNotSupportedError(ApiException):
    """
    An exception that will result in a response status of 405.

    Raised if user tries to DELETE, POST or PUT for an Object
    where we don't support those methods.
    """

    status = 405


class CreatedObject(Exception):
    """
    An exception that is thrown when new object created.

    This is not really an error but indicates to the handler
    that a JsonResponse with status 201 should be returned.
    The dict content is passed in as 'response'.
    """

    status = 201

    def __init__(self, response):
        """Override init to include response dict."""
        super(CreatedObject, self).__init__(response)
        self.response = response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging

logger = logging.getLogger(__name__)


def upgradeCheck(url):
    # upgrade check:
    # -------------
    # On each startup OMERO.web checks for possible server upgrades
    # and logs the upgrade url at the WARNING level. If you would
    # like to disable the checks, please set 'omero.web.upgrades_url`
    # to an empty string.
    #
    # For more information, see
    # https://docs.openmicroscopy.org/latest/omero/sysadmins/UpgradeCheck.html
    #
    try:
        from omero.util.upgrade_check import UpgradeCheck
        if url:
            check = UpgradeCheck("web", url=url)
            check.run()
            if check.isUpgradeNeeded():
                logger.warn(
                    "Upgrade is available. Please visit"
                    " https://downloads.openmicroscopy.org/latest/omero/.\n")
            else:
                logger.debug("Up to date.\n")
    except Exception as x:
        logger.error("Upgrade check error: %s" % x)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2008-2014 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile
import zipfile
import shutil
import logging
try:
    import long
except ImportError:
    long = int

logger = logging.getLogger(__name__)

LUTS_IN_PNG = [
    '/luts/ncsa_paledit/16_colors.lut',
    '/luts/3-3-2_rgb.lut',
    '/luts/ncsa_paledit/5_ramps.lut',
    '/luts/ncsa_paledit/6_shades.lut',
    '/luts/ncsa_paledit/blue_orange_icb.lut',
    '/luts/ncsa_paledit/brgbcmyw.lut',
    '/luts/ncsa_paledit/cool.lut',
    '/luts/wcif/cyan_hot.lut',
    '/luts/ncsa_paledit/edges.lut',
    '/luts/fire.lut',
    '/luts/ncsa_paledit/gem.lut',
    '/luts/glasbey.lut',
    '/luts/glasbey_inverted.lut',
    '/luts/glow.lut',
    '/luts/grays.lut',
    '/luts/wcif/green_fire_blue.lut',
    '/luts/wcif/hilo.lut',
    '/luts/wcif/ica.lut',
    '/luts/wcif/ica2.lut',
    '/luts/wcif/ica3.lut',
    '/luts/ice.lut',
    '/luts/wcif/magenta_hot.lut',
    '/luts/wcif/orange_hot.lut',
    '/luts/ncsa_paledit/phase.lut',
    '/luts/physics.lut',
    '/luts/janelia/pup_br.lut',
    '/luts/janelia/pup_nr.lut',
    '/luts/wcif/rainbow_rgb.lut',
    '/luts/red-green.lut',
    '/luts/wcif/red_hot.lut',
    '/luts/ncsa_paledit/royal.lut',
    '/luts/ncsa_paledit/sepia.lut',
    '/luts/ncsa_paledit/smart.lut',
    '/luts/spectrum.lut',
    '/luts/ncsa_paledit/thal.lut',
    '/luts/ncsa_paledit/thallium.lut',
    '/luts/thermal.lut',
    '/luts/ncsa_paledit/unionjack.lut',
    '/luts/wcif/yellow_hot.lut',
    # NB: this is not a LUT but it is in the current png
    # Used for channel slider gradient when no lut chosen
    'gradient.png',
]


# helper method
def getIntOrDefault(request, name, default):
    index = request.GET.get(name, request.POST.get(name, default))
    if index is not None:
        index = int(index)
    return index


def get_longs(request, name):
    """
    Retrieves parameters from the request. If the parameters are not present
    an empty list is returned

    This does not catch exceptions as it makes sense to throw exceptions if
    the arguments provided do not pass basic type validation
    """
    vals = []
    vals_raw = request.GET.getlist(name)
    for val_raw in vals_raw:
        vals.append(long(val_raw))
    return vals


def zip_archived_files(images, temp, zipName, buf=2621440):
    """
    Util function to download original files from a list of images
    and arrange them within a temp file, such that there are no
    name clashes and multi-image filesets are kept distict.
    Handles archived files from OMERO 4 images and fileset files
    for OMERO 5 images.

    @param images:      Images as source of original files.
    @type               List of ImageWrappers
    @param temp:        Directory for creating Zip file
    @param zipName:     Name of zip
    """

    # ',' in name causes duplicate headers
    zipName = zipName.replace(" ", "_").replace(",", ".")
    if not zipName.endswith('.zip'):
        zipName = "%s.zip" % zipName

    def getTargetPath(fsFile, templatePrefix):
        if fsFile.getPath() == templatePrefix or templatePrefix == "":
            return fsFile.getName()
        relPath = os.path.relpath(fsFile.getPath(), templatePrefix)
        return os.path.join(relPath, fsFile.getName())

    def split_path(p):
        a, b = os.path.split(p)
        return (split_path(a) if len(a) and len(b) else []) + [b]

    fsIds = set()
    fIds = set()

    temp_zip_dir = tempfile.mkdtemp()
    logger.debug("download dir: %s" % temp_zip_dir)
    try:
        new_dir_idx = 1     # if needed to avoid file name clashes
        for image in images:
            new_dir = ""
            templatePrefix = ""
            fs = image.getFileset()
            if fs is not None:
                # Make sure we've not processed this fileset before.
                if fs.id in fsIds:
                    continue
                fsIds.add(fs.id)
                templatePrefix = fs.getTemplatePrefix()
            files = list(image.getImportedImageFiles())

            # check if ANY of the files will overwrite exising file
            for f in files:
                target_path = getTargetPath(f, templatePrefix)
                base_file = os.path.join(
                    temp_zip_dir, split_path(target_path)[0])
                if os.path.exists(base_file):
                    new_dir = str(new_dir_idx)
                    new_dir_idx += 1
                    break

            for a in files:
                # check for duplicate files for OMERO 4.4 images (no fileset)
                if a.id in fIds:
                    continue
                fIds.add(a.id)
                temp_f = getTargetPath(a, templatePrefix)
                temp_f = os.path.join(temp_zip_dir, new_dir, temp_f)
                temp_d = os.path.dirname(temp_f)
                if not os.path.exists(temp_d):
                    os.makedirs(temp_d)

                # Need to be sure that the zip name does not match any file
                # within it since OS X will unzip as a single file instead of
                # a directory
                if zipName == "%s.zip" % a.name:
                    zipName = "%s_folder.zip" % a.name

                f = open(str(temp_f), "wb")
                try:
                    for chunk in a.getFileInChunks(buf=buf):
                        f.write(chunk)
                finally:
                    f.close()

        # create zip
        zip_file = zipfile.ZipFile(temp, 'w', zipfile.ZIP_DEFLATED)
        try:
            for root, dirs, files in os.walk(temp_zip_dir):
                archive_root = os.path.relpath(root, temp_zip_dir)
                for f in files:
                    fullpath = os.path.join(root, f)
                    archive_name = os.path.join(archive_root, f)
                    zip_file.write(fullpath, archive_name)
        finally:
            zip_file.close()
            # delete temp dir
    finally:
        shutil.rmtree(temp_zip_dir, ignore_errors=True)

    return zipName


def xy_list_to_bbox(xyList):
    """
    Returns a bounding box (x,y,w,h) that will contain the shape
    represented by the XY points list
    """
    xList, yList = [], []
    for xy in xyList:
        x, y = xy
        xList.append(x)
        yList.append(y)
    return (min(xList), min(yList), max(xList)-min(xList),
            max(yList)-min(yList))


def points_string_to_XY_list(string):
    """
    Method for converting the string returned from
    omero.model.ShapeI.getPoints() into list of (x,y) points.
    E.g: "points[309,427, 366,503, 190,491] points1[309,427, 366,503,
    190,491] points2[309,427, 366,503, 190,491]"
    or the new format: "309,427 366,503 190,491"
    OR with extra ,  : "309,427, 366,503, 190,491"
    """
    pointLists = string.strip().split("points")
    if len(pointLists) < 2:
        if len(pointLists) == 1 and pointLists[0]:
            xys = pointLists[0].split(" ")
            xyList = [tuple(map(float, xy.strip(',').split(',')))
                      for xy in xys if len(xy) > 0]
            return xyList

        msg = "Unrecognised ROI shape 'points' string: %s" % string
        raise ValueError(msg)

    firstList = pointLists[1]
    xyList = []
    for xy in firstList.strip(" []").split(", "):
        x, y = xy.split(",")
        xyList.append((float(x.strip()), float(y.strip())))
    return xyList



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (c) 2014 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Generic functionality for handling particular links and "showing" objects
in the OMERO.web tree view.
"""

import omero
import re

from omero.rtypes import rint, rlong
from django.core.urlresolvers import reverse
from copy import deepcopy
from django.conf import settings


class IncorrectMenuError(Exception):
    """Exception to signal that we are on the wrong menu."""

    def __init__(self, uri):
        """
        Constructs a new Exception instance.

        @param uri URI to redirect to.
        @type uri String
        """
        super(Exception, self).__init__()
        self.uri = uri


class Show(object):
    """
    This object is used by most of the top-level pages.  The "show" and
    "path" query strings are used by this object to both direct OMERO.web to
    the correct locations in the hierarchy and select the correct objects
    in that hierarchy.
    """

    # List of prefixes that are at the top level of the tree
    TOP_LEVEL_PREFIXES = ('project', 'screen', 'tagset')

    # List of supported object types
    SUPPORTED_OBJECT_TYPES = (
        'project', 'dataset', 'image', 'screen', 'plate', 'tag',
        'acquisition', 'run', 'well', 'tagset', 'roi'
    )

    # Regular expression which declares the format for a "path" used either
    # in the "path" or "show" query string.  No modifications should be made
    # to this regex without corresponding unit tests in
    # "tests/unit/test_show.py".
    PATH_REGEX = re.compile(
        r'(?P<object_type>\w+)\.?(?P<key>\w+)?[-=](?P<value>[^\|]*)\|?'
    )

    # Regular expression for matching Well names
    WELL_REGEX = re.compile(
        r'^(?:(?P<alpha_row>[a-zA-Z]+)(?P<digit_column>\d+))|'
        r'(?:(?P<digit_row>\d+)(?P<alpha_column>[a-zA-Z]+))$'
    )

    def __init__(self, conn, request, menu):
        """
        Constructs a Show instance.  The instance will not be fully
        initialised until the first retrieval of the L{Show.first_selected}
        property.

        @param conn OMERO gateway.
        @type conn L{omero.gateway.BlitzGateway}
        @param request Django HTTP request.
        @type request L{django.http.HttpRequest}
        @param menu Literal representing the current menu we are on.
        @type menu String
        """
        # The list of "paths" ("type-id") we have been requested to
        # show/select in the user interface.  May be modified if one or
        # more of the elements is not in the tree.  This is currently the
        # case for all Screen-Plate-Well hierarchy elements below Plate
        # (Well for example).
        self._initially_select = list()
        # The nodes of the tree that will be initially open based on the
        # nodes that are initially selected.
        self._initially_open = list()
        # The owner of the node closest to the root of the tree from the
        # list of initially open nodes.
        self._initially_open_owner = None
        # First selected node from the requested initially open "paths"
        # that is first loaded on first retrieval of the "first_selected"
        # property.
        self._first_selected = None

        self.conn = conn
        self.request = request
        self.menu = menu

        path = self.request.GET.get('path', '').split('|')[-1]
        self._add_if_supported(path)

        show = self.request.GET.get('show', '')
        for path in show.split('|'):
            self._add_if_supported(path)

    def _add_if_supported(self, path):
        """Adds a path to the initially selected list if it is supported."""
        m = self.PATH_REGEX.match(path)
        if m is None:
            return
        object_type = m.group('object_type')
        key = m.group('key')
        value = m.group('value')
        if key is None:
            key = 'id'
        if object_type in self.SUPPORTED_OBJECT_TYPES:
            # 'run' is an alternative for 'acquisition'
            object_type = object_type.replace('run', 'acquisition')
            self._initially_select.append(
                '%s.%s-%s' % (object_type, key, value)
            )

    def _load_tag(self, attributes):
        """
        Loads a Tag based on a certain set of attributes from the server.

        @param attributes Set of attributes to filter on.
        @type attributes L{dict}
        """
        # Tags have an "Annotation" suffix added to the object name so
        # need to be loaded differently.
        return next(self.conn.getObjects(
            "TagAnnotation", attributes=attributes
        ))

    def get_well_row_column(self, well):
        """
        Retrieves a tuple of row and column as L{int} for a given Well name
        ("A1" or "1A") string.

        @param well Well name string to retrieve the row and column tuple for.
        @type well L{str}
        """
        m = self.WELL_REGEX.match(well)
        if m is None:
            return None
        # We are using an algorithm that expects alpha columns and digit
        # rows (like a spreadsheet).  is_reversed will be True if those
        # conditions are not met, signifying that the row and column
        # calculated needs to be reversed before returning.
        is_reversed = False
        if m.group('alpha_row') is not None:
            a = m.group('alpha_row').upper()
            b = m.group('digit_column')
            is_reversed = True
        else:
            a = m.group('alpha_column').upper()
            b = m.group('digit_row')

        # Convert base26 column string to number.  Adapted from XlsxWriter:
        #   * https://github.com/jmcnamara/XlsxWriter
        #     * xlsxwriter/utility.py
        n = 0
        column = 0
        for character in reversed(a):
            column += (ord(character) - ord('A') + 1) * (26 ** n)
            n += 1

        # Convert 1-index to zero-index
        row = int(b) - 1
        column -= 1

        if is_reversed:
            return column, row
        return row, column

    def _load_well(self, attributes):
        """
        Loads a Well based on a certain set of attributes from the server.

        @param attributes Set of attributes to filter on.
        @type attributes L{dict}
        """
        if 'id' in attributes:
            return self.conn.getObject('Well', attributes=attributes)
        if 'name' in attributes:
            row, column = self.get_well_row_column(attributes['name'])
            path = self.request.GET.get('path', '')
            for m in self.PATH_REGEX.finditer(path):
                object_type = m.group('object_type')
                # May have 'run' here rather than 'acquisition' because
                # the path will not have been validated and replaced.
                if object_type not in ('plate', 'run', 'acquisition'):
                    continue
                # 'run' is an alternative for 'acquisition'
                object_type = object_type.replace('run', 'acquisition')

                # Try and load the potential parent first
                key = m.group('key')
                value = m.group('value')
                if key is None:
                    key = 'id'
                if key == 'id':
                    value = int(value)
                parent_attributes = {key: value}
                parent, = self.conn.getObjects(
                    object_type, attributes=parent_attributes
                )

                # Now use the parent to try and locate the Well
                query_service = self.conn.getQueryService()
                params = omero.sys.ParametersI()
                params.map['row'] = rint(row)
                params.map['column'] = rint(column)
                params.addId(parent.id)
                if object_type == 'plate':
                    db_row, = query_service.projection(
                        'select w.id from Well as w '
                        'where w.row = :row and w.column = :column '
                        'and w.plate.id = :id', params, self.conn.SERVICE_OPTS
                    )
                if object_type == 'acquisition':
                    db_row, = query_service.projection(
                        'select distinct w.id from Well as w '
                        'join w.wellSamples as ws '
                        'where w.row = :row and w.column = :column '
                        'and ws.plateAcquisition.id = :id',
                        params, self.conn.SERVICE_OPTS
                    )
                well_id, = db_row
                return self.conn.getObject(
                    'Well', well_id.val
                )

    def _load_first_selected(self, first_obj, attributes):
        """
        Loads the first selected object from the server.  Will raise
        L{IncorrectMenuError} if the initialized menu was incorrect for
        the loaded objects.

        @param first_obj Type of the first selected object.
        @type first_obj String
        @param attributes Set of attributes to filter on.
        @type attributes L{dict}
        """
        first_selected = None
        if first_obj in ["tag", "tagset"]:
            first_selected = self._load_tag(attributes)
        elif first_obj == "well":
            first_selected = self._load_well(attributes)
        else:
            # All other objects can be loaded by type and attributes.
            first_selected, = self.conn.getObjects(
                first_obj, attributes=attributes
            )

        if first_obj == "well":
            # Wells aren't in the tree, so we need to look up the parent
            well_sample = first_selected.getWellSample()
            parent_node = None
            parent_type = None
            # It's possible that the Well that we've been requested to show
            # has no fields (WellSample instances).  In that case the Plate
            # will be used but we don't have much choice.
            if well_sample is not None:
                parent_node = well_sample.getPlateAcquisition()
                parent_type = "acquisition"
            if parent_node is None:
                # No WellSample for this well, try and retrieve the
                # PlateAcquisition from the parent Plate.
                plate = first_selected.getParent()
                try:
                    parent_node, = plate.listPlateAcquisitions()
                    parent_type = "acquisition"
                except ValueError:
                    # No PlateAcquisition for this well, use Plate instead
                    parent_node = plate
                    parent_type = "plate"
            # Tree hierarchy open to first selected "real" object available
            # in the tree.
            self._initially_open = [
                "%s-%s" % (parent_type, parent_node.getId()),
                "%s-%s" % (first_obj, first_selected.getId())
            ]
            first_selected = parent_node
        else:
            # If show=roi, selected object will be Image
            if first_obj == "roi":
                first_obj = "image"
                first_selected = first_selected.getImage()

            # Tree hierarchy open to first selected object.
            self._initially_open = [
                '%s-%s' % (first_obj, first_selected.getId())
            ]
        # support for multiple objects selected by ID,
        # E.g. show=image-1|image-2
        if 'id' in attributes.keys() and len(self._initially_select) > 1:
            # 'image.id-1' -> 'image-1'
            self._initially_select = [
                i.replace(".id", "") for i in self._initially_select]
        else:
            # Only select a single object
            self._initially_select = self._initially_open[:]
        self._initially_open_owner = first_selected.details.owner.id.val
        return first_selected

    def _find_first_selected(self):
        """Finds the first selected object."""
        if len(self._initially_select) == 0:
            return None

        # tree hierarchy open to first selected object
        m = self.PATH_REGEX.match(self._initially_select[0])
        if m is None:
            return None
        first_obj = m.group('object_type')
        # if we're showing a tag, make sure we're on the tags page...
        if first_obj in ["tag", "tagset"] and self.menu != "usertags":
            # redirect to usertags/?show=tag-123
            raise IncorrectMenuError(
                reverse(viewname="load_template", args=['usertags']) +
                "?show=" + self._initially_select[0].replace(".id", "")
            )
        first_selected = None
        try:
            key = m.group('key')
            value = m.group('value')
            if key == 'id':
                value = int(value)
            attributes = {key: value}
            # Set context to 'cross-group'
            self.conn.SERVICE_OPTS.setOmeroGroup('-1')
            first_selected = self._load_first_selected(first_obj, attributes)
        except Exception:
            pass
        if first_obj not in self.TOP_LEVEL_PREFIXES:
            # Need to see if first item has parents
            if first_selected is not None:
                for p in first_selected.getAncestry():
                    # If 'Well' is a parent, we have stared with Image.
                    # We want to start again at 'Well' to _load_first_selected
                    # with well, so we get 'acquisition' in ancestors.
                    if p.OMERO_CLASS == "Well":
                        self._initially_select = ['well.id-%s' % p.getId()]
                        return self._find_first_selected()
                    if first_obj == "tag":
                        # Parents of tags must be tagset (no OMERO_CLASS)
                        self._initially_open.insert(0, "tagset-%s" % p.getId())
                    else:
                        self._initially_open.insert(
                            0, "%s-%s" % (p.OMERO_CLASS.lower(), p.getId())
                        )
                    self._initially_open_owner = p.details.owner.id.val
                m = self.PATH_REGEX.match(self._initially_open[0])
                if m.group('object_type') == 'image':
                    self._initially_open.insert(0, "orphaned-0")
        return first_selected

    @property
    def first_selected(self):
        """
        Retrieves the first selected object.  The first time this method is
        invoked on the instance the actual retrieval is performed.  All other
        invocations retrieve the same instance without server interaction.
        Will raise L{IncorrectMenuError} if the initialized menu was
        incorrect for the loaded objects.
        """
        if self._first_selected is None:
            self._first_selected = self._find_first_selected()
        return self._first_selected

    @property
    def initially_select(self):
        """
        Retrieves the list of "paths" ("type-id") we have been requested to
        show/select in the user interface.  May be different than we were
        first initialised with due to certain nodes of the Screen-Plate-Well
        hierachy not being present in the tree.  Should not be invoked until
        after first retrieval of the L{Show.first_selected} property.
        """
        return self._initially_select

    @property
    def initially_open(self):
        """
        Retrieves the nodes of the tree that will be initially open based on
        the nodes that are initially selected.  Should not be invoked until
        after first retrieval of the L{Show.first_selected} property.
        """
        return self._initially_open

    @property
    def initially_open_owner(self):
        """
        Retrieves the owner of the node closest to the root of the tree from
        the list of initially open nodes.  Should not be invoked until
        after first retrieval of the L{Show.first_selected} property.
        """
        return self._initially_open_owner


def get_image_ids(conn, datasetId=None, groupId=-1, ownerId=None):
    """
    Retrieves a list of all image IDs in a Dataset or Orphaned
    (with owner specified by ownerId). The groupId can be specified
    as needed, particuarly when querying orphaned images.
    """
    qs = conn.getQueryService()
    p = omero.sys.ParametersI()
    so = deepcopy(conn.SERVICE_OPTS)
    so.setOmeroGroup(groupId)
    if datasetId is not None:
        p.add('did', rlong(datasetId))
        q = """select image.id from Image image
            join image.datasetLinks dlink where dlink.parent.id = :did
            order by lower(image.name), image.id"""
    else:
        p.add('ownerId', rlong(ownerId))
        q = """select image.id from Image image where
            image.details.owner.id = :ownerId and
            not exists (
                select dilink from DatasetImageLink as dilink
                where dilink.child = image.id

            )  and
            not exists (
                select ws from WellSample ws
                where ws.image.id = image.id
            )
            order by lower(image.name), image.id"""
    iids = [i[0].val for i in qs.projection(q, p, so)]
    return iids


def paths_to_object(conn, experimenter_id=None, project_id=None,
                    dataset_id=None, image_id=None, screen_id=None,
                    plate_id=None, acquisition_id=None, well_id=None,
                    group_id=None, page_size=None):
    """
    Retrieves the parents of an object (E.g. P/D/I for image) as a list
    of paths.
    Lowest object in hierarchy is found by checking parameter ids in order:
    image->dataset->project->well->acquisition->plate->screen->experimenter
    If object has multiple paths, these can also be filtered by parent_ids.
    E.g. paths to image_id filtered by dataset_id.

    If image is in a Dataset or Orphaned collection that is paginated
    (imageCount > page_size) then we include 'childPage', 'childCount'
    and 'childIndex' in the dataset or orphaned dict.
    The page_size default is settings.PAGE (omero.web.page_size)

    Note on wells:
    Selecting a 'well' is really for selecting well_sample paths
    if a well is specified on its own, we return all the well_sample paths
    than match
    """

    qs = conn.getQueryService()
    if page_size is None:
        page_size = settings.PAGE

    params = omero.sys.ParametersI()
    service_opts = deepcopy(conn.SERVICE_OPTS)

    lowest_type = None
    if experimenter_id is not None:
        params.add('eid', rlong(experimenter_id))
        lowest_type = 'experimenter'
    if screen_id is not None:
        params.add('sid', rlong(screen_id))
        lowest_type = 'screen'
    if plate_id is not None:
        params.add('plid', rlong(plate_id))
        lowest_type = 'plate'
    if acquisition_id is not None:
        params.add('aid', rlong(acquisition_id))
        lowest_type = 'acquisition'
    if well_id is not None:
        params.add('wid', rlong(well_id))
        lowest_type = 'well'
    if project_id is not None:
        params.add('pid', rlong(project_id))
        lowest_type = 'project'
    if dataset_id is not None:
        params.add('did', rlong(dataset_id))
        lowest_type = 'dataset'
    if image_id is not None:
        params.add('iid', rlong(image_id))
        lowest_type = 'image'
    # If none of these parameters are set then there is nothing to find
    if lowest_type is None:
        return []

    if group_id is not None:
        service_opts.setOmeroGroup(group_id)

    # Hierarchies for this object
    paths = []

    # It is probably possible to write a more generic query instead
    # of special casing each type, but it will be less readable and
    # maintainable than these

    if lowest_type == 'image':
        q = '''
            select coalesce(powner.id, downer.id, iowner.id),
                   pdlink.parent.id,
                   dilink.parent.id,
                   (select count(id) from DatasetImageLink dil
                    where dil.parent=dilink.parent.id),
                   image.id,
                   image.details.group.id as groupId
            from Image image
            join image.details.owner iowner
            left outer join image.datasetLinks dilink
            left outer join dilink.parent.details.owner downer
            left outer join dilink.parent.projectLinks pdlink
            left outer join pdlink.parent.details.owner powner
            where image.id = :iid
            '''
        where_clause = []
        if dataset_id is not None:
            where_clause.append('dilink.parent.id = :did')
        if project_id is not None:
            where_clause.append('pdlink.parent.id = :pid')
        if experimenter_id is not None:
            where_clause.append(
                'coalesce(powner.id, downer.id, iowner.id) = :eid')
        if len(where_clause) > 0:
            q += ' and ' + ' and '.join(where_clause)

        q += '''
             order by coalesce(powner.id, downer.id, iowner.id),
                      pdlink.parent.id,
                      dilink.parent.id,
                      image.id
             '''

        for e in qs.projection(q, params, service_opts):
            path = []
            imageId = e[4].val

            # Experimenter is always found
            path.append({
                'type': 'experimenter',
                'id': e[0].val
            })

            # If it is experimenter->project->dataset->image
            if e[1] is not None:
                path.append({
                    'type': 'project',
                    'id': e[1].val
                })

            # If it is experimenter->dataset->image or
            # experimenter->project->dataset->image
            if e[2] is not None:
                imgCount = e[3].val
                datasetId = e[2].val
                ds = {
                    'type': 'dataset',
                    'id': datasetId,
                    'childCount': imgCount,
                }
                if imgCount > page_size:
                    # Need to know which page image is on
                    iids = get_image_ids(conn, datasetId)
                    index = iids.index(imageId)
                    page = (index // page_size) + 1  # 1-based index
                    ds['childIndex'] = index
                    ds['childPage'] = page
                path.append(ds)

            # If it is orphaned->image
            paths_to_img = []
            if e[2] is None:
                # Check if image is in Well
                paths_to_img = paths_to_well_image(
                    conn, params,
                    well_id=well_id, image_id=image_id,
                    acquisition_id=acquisition_id,
                    plate_id=plate_id,
                    screen_id=screen_id,
                    experimenter_id=experimenter_id,
                    orphanedImage=True)
                if len(paths_to_img) == 0:
                    orph = {
                        'type': 'orphaned',
                        'id': e[0].val
                    }
                    iids = get_image_ids(conn, groupId=e[5].val,
                                         ownerId=e[0].val)
                    if len(iids) > page_size:
                        try:
                            index = iids.index(imageId)
                            page = (index // page_size) + 1  # 1-based index
                            orph['childCount'] = len(iids)
                            orph['childIndex'] = index
                            orph['childPage'] = page
                        except ValueError:
                            # If image is in Well, it won't be in orphaned list
                            pass
                    path.append(orph)

            if len(paths_to_img) > 0:
                paths = paths_to_img
            else:
                # Image always present
                path.append({
                    'type': 'image',
                    'id': imageId
                })
                paths.append(path)

    elif lowest_type == 'dataset':
        q = '''
            select coalesce(powner.id, downer.id),
                   pdlink.parent.id,
                   dataset.id
            from Dataset dataset
            join dataset.details.owner downer
            left outer join dataset.projectLinks pdlink
            left outer join pdlink.parent.details.owner powner
            where dataset.id = :did
            '''
        where_clause = []
        if project_id is not None:
            where_clause.append('pdlink.parent.id = :pid')
        if experimenter_id is not None:
            where_clause.append('coalesce(powner.id, downer.id) = :eid')
        if len(where_clause) > 0:
            q += ' and ' + ' and '.join(where_clause)

        for e in qs.projection(q, params, service_opts):
            path = []

            # Experimenter is always found
            path.append({
                'type': 'experimenter',
                'id': e[0].val
            })

            # If it is experimenter->project->dataset
            if e[1] is not None:
                path.append({
                    'type': 'project',
                    'id': e[1].val
                })

            # Dataset always present
            path.append({
                'type': 'dataset',
                'id': e[2].val
            })

            paths.append(path)

    elif lowest_type == 'project':
        q = '''
            select project.details.owner.id,
                   project.id
            from Project project
            where project.id = :pid
            '''

        for e in qs.projection(q, params, service_opts):
            path = []

            # Always experimenter->project
            path.append({
                'type': 'experimenter',
                'id': e[0].val
            })

            path.append({
                'type': 'project',
                'id': e[1].val
            })

            paths.append(path)

    # This is basically the same as WellSample except that it is not
    # restricted by a particular WellSample id
    # May not have acquisition (load plate from well)
    # We don't need to load the wellsample (not in tree)
    elif lowest_type == 'well':

        paths_to_img = paths_to_well_image(conn, params,
                                           well_id=well_id,
                                           image_id=image_id,
                                           acquisition_id=acquisition_id,
                                           plate_id=plate_id,
                                           screen_id=screen_id,
                                           experimenter_id=experimenter_id)
        if len(paths_to_img) > 0:
            paths.extend(paths_to_img)

    elif lowest_type == 'acquisition':
        q = '''
            select coalesce(sowner.id, plowner.id, aowner.id),
                   slink.parent.id,
                   plate.id,
                   acquisition.id
            from PlateAcquisition acquisition
            join acquisition.details.owner aowner
            left outer join acquisition.plate plate
            left outer join plate.details.owner plowner
            left outer join plate.screenLinks slink
            left outer join slink.parent.details.owner sowner
            where acquisition.id = :aid
            '''
        where_clause = []
        if plate_id is not None:
            where_clause.append('plate.id = :plid')
        if screen_id is not None:
            where_clause.append('slink.parent.id = :sid')
        if experimenter_id is not None:
            where_clause.append(
                'coalesce(sowner.id, plowner.id, aowner.id) = :eid')
        if len(where_clause) > 0:
            q += ' and ' + ' and '.join(where_clause)

        for e in qs.projection(q, params, service_opts):
            path = []

            # Experimenter is always found
            path.append({
                'type': 'experimenter',
                'id': e[0].val
            })

            # If it is experimenter->screen->plate->acquisition
            if e[1] is not None:
                path.append({
                    'type': 'screen',
                    'id': e[1].val
                })

            # If it is experimenter->plate->acquisition or
            # experimenter->screen->plate->acquisition
            if e[2] is not None:
                path.append({
                    'type': 'plate',
                    'id': e[2].val
                })

            # Acquisition always present
            path.append({
                'type': 'acquisition',
                'id': e[3].val
            })

            paths.append(path)

    elif lowest_type == 'plate':
        q = '''
            select coalesce(sowner.id, plowner.id),
                   splink.parent.id,
                   plate.id
            from Plate plate
            join plate.details.owner sowner
            left outer join plate.screenLinks splink
            left outer join splink.parent.details.owner plowner
            where plate.id = :plid
            '''
        where_clause = []
        if screen_id is not None:
            where_clause.append('splink.parent.id = :sid')
        if experimenter_id is not None:
            where_clause.append('coalesce(sowner.id, plowner.id) = :eid')
        if len(where_clause) > 0:
            q += ' and ' + ' and '.join(where_clause)

        for e in qs.projection(q, params, service_opts):
            path = []

            # Experimenter is always found
            path.append({
                'type': 'experimenter',
                'id': e[0].val
            })

            # If it is experimenter->screen->plate
            if e[1] is not None:
                path.append({
                    'type': 'screen',
                    'id': e[1].val
                })

            # Plate always present
            path.append({
                'type': 'plate',
                'id': e[2].val
            })

            paths.append(path)

    elif lowest_type == 'screen':
        q = '''
            select screen.details.owner.id,
                   screen.id
            from Screen screen
            where screen.id = :sid
            '''

        for e in qs.projection(q, params, service_opts):
            path = []

            # Always experimenter->screen
            path.append({
                'type': 'experimenter',
                'id': e[0].val
            })

            path.append({
                'type': 'screen',
                'id': e[1].val
            })

            paths.append(path)

    elif lowest_type == 'experimenter':
        path = []

        # No query required here as this is the highest level container
        path.append({
            'type': 'experimenter',
            'id': experimenter_id
        })

        paths.append(path)

    return paths


def paths_to_well_image(conn, params, well_id=None, image_id=None,
                        acquisition_id=None,
                        plate_id=None, screen_id=None, experimenter_id=None,
                        orphanedImage=False):

    qs = conn.getQueryService()
    service_opts = deepcopy(conn.SERVICE_OPTS)
    q = '''
        select coalesce(sowner.id, plowner.id, aowner.id, wsowner.id),
               slink.parent.id,
               plate.id,
               acquisition.id,
               well.id
        from WellSample wellsample
        join wellsample.details.owner wsowner
        left outer join wellsample.plateAcquisition acquisition
        left outer join acquisition.details.owner aowner
        join wellsample.well well
        join well.plate plate
        join plate.details.owner plowner
        left outer join plate.screenLinks slink
        left outer join slink.parent.details.owner sowner
        '''
    where_clause = []
    if well_id is not None:
        where_clause.append('wellsample.well.id = :wid')
    if image_id is not None:
        where_clause.append('wellsample.image.id = :iid')
    if acquisition_id is not None:
        where_clause.append('acquisition.id = :aid')
    if plate_id is not None:
        where_clause.append('plate.id = :plid')
    if screen_id is not None:
        where_clause.append('slink.parent.id = :sid')
    if experimenter_id is not None:
        where_clause.append(
            'coalesce(sowner.id, plowner.id, aoener.id, wowner.id) = :eid')
    if len(where_clause) > 0:
        q += 'where ' + ' and '.join(where_clause)

    paths = []
    for e in qs.projection(q, params, service_opts):
        path = []

        # Experimenter is always found
        path.append({
            'type': 'experimenter',
            'id': e[0].val
        })

        # If it is experimenter->screen->plate->acquisition->wellsample
        if e[1] is not None:
            path.append({
                'type': 'screen',
                'id': e[1].val
            })

        # Plate should always present
        path.append({
            'type': 'plate',
            'id': e[2].val
        })

        # Acquisition not present if plate created via API (not imported)
        if e[3] is not None:
            path.append({
                'type': 'acquisition',
                'id': e[3].val
            })

        # Include Well if path is to image
        if e[4] is not None and orphanedImage:
            path.append({
                'type': 'well',
                'id': e[4].val
            })

        paths.append(path)
    return paths


def paths_to_tag(conn, experimenter_id=None, tagset_id=None, tag_id=None):
    """
    Finds tag for tag_id, also looks for parent tagset in path.
    If tag_id and tagset_id are given, only return paths that have both.
    If no tagset/tag paths are found, simply look for tags with tag_id.
    """
    params = omero.sys.ParametersI()
    service_opts = deepcopy(conn.SERVICE_OPTS)
    where_clause = []

    if experimenter_id is not None:
        params.add('eid', rlong(experimenter_id))
        where_clause.append(
            'coalesce(tsowner.id, towner.id) = :eid')

    if tag_id is not None:
        params.add('tid', rlong(tag_id))
        where_clause.append(
            'ttlink.child.id = :tid')

    if tagset_id is not None:
        params.add('tsid', rlong(tagset_id))
        where_clause.append(
            'tagset.id = :tsid')

    if tag_id is None and tagset_id is None:
        return []

    qs = conn.getQueryService()
    paths = []

    # Look for tag in a tagset...
    if tag_id is not None:
        q = '''
            select coalesce(tsowner.id, towner.id),
                   tagset.id,
                   ttlink.child.id
            from TagAnnotation tagset
            join tagset.details.owner tsowner
            left outer join tagset.annotationLinks ttlink
            left outer join ttlink.child.details.owner towner
            where %s
        ''' % ' and '.join(where_clause)

        tagsets = qs.projection(q, params, service_opts)
        for e in tagsets:
            path = []
            # Experimenter is always found
            path.append({
                'type': 'experimenter',
                'id': e[0].val
            })
            path.append({
                'type': 'tagset',
                'id': e[1].val
            })
            path.append({
                'type': 'tag',
                'id': e[2].val
            })
            paths.append(path)

    # If we haven't found tag in tagset, just look for tags with matching IDs
    if len(paths) == 0:

        where_clause = []

        if experimenter_id is not None:
            # params.add('eid', rlong(experimenter_id))
            where_clause.append(
                'coalesce(tsowner.id, towner.id) = :eid')

        if tag_id is not None:
            # params.add('tid', rlong(tag_id))
            where_clause.append(
                'tag.id = :tid')

        elif tagset_id is not None:
            # params.add('tsid', rlong(tagset_id))
            where_clause.append(
                'tag.id = :tsid')

        q = '''
            select towner.id, tag.id
            from TagAnnotation tag
            left outer join tag.details.owner towner
            where %s
        ''' % ' and '.join(where_clause)

        tagsets = qs.projection(q, params, service_opts)
        for e in tagsets:
            path = []
            # Experimenter is always found
            path.append({
                'type': 'experimenter',
                'id': e[0].val
            })
            if tag_id is not None:
                t = 'tag'
            else:
                t = 'tagset'
            path.append({
                'type': t,
                'id': e[1].val
            })
            paths.append(path)

    return paths



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# #!/usr/bin/env python
#
#
#
# Copyright (c) 2008-2013 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2008.
#
# Version: 1.0
#


import datetime
import traceback
import logging
import json
import random

from past.builtins import basestring

from django import template

register = template.Library()

logger = logging.getLogger(__name__)


@register.filter()
def jsonify(obj):
    """Simple template filter to encode a variable to JSON format"""
    return json.dumps(obj)


@register.filter
def hash(value, key):
    return value[key]


@register.filter
def random_if_none(value):
    if value is None:
        value = str(random.random())[2:]
    return value


@register.filter
def random_if_minus_one(value):
    """ Used for thumbnail versions """
    if value == -1:
        value = str(random.random())[2:]
    return value


@register.filter
def ago(value):
    """
    Formats a datetime.datetime object as time Ago.
    E.g. '3 days 2 hours 10 minutes'
    """
    try:
        ago = datetime.datetime.now() - value
    except TypeError:
        return str(value)

    def plurals(val):
        return val != 1 and "s" or ""
    hours, remainder = divmod(ago.seconds, 3600)
    mins, secs = divmod(remainder, 60)
    if ago.days >= 365:
        years = ago.days / 365
        return "%s year%s" % (years, plurals(years))
    if ago.days > 28:
        months = ago.days / 30
        return "%s month%s" % (months, plurals(months))
    if ago.days > 0:
        return "%s day%s" % (ago.days, plurals(ago.days))
    if hours > 0:
        return "%s hour%s" % (hours, plurals(hours))
    if mins > 1:
        return "%s minutes" % (mins)
    if mins == 1:
        return "a minute"
    return "less than a minute"


@register.filter
def truncateafter(value, arg):
    """
    Truncates a string after a given number of chars
    Argument: Number of chars to truncate after
    """
    try:
        length = int(arg)
    except ValueError:  # invalid literal for int()
        return value  # Fail silently.
    if not isinstance(value, basestring):
        value = str(value)
    if (len(value) > length):
        return value[:length] + "..."
    else:
        return value


@register.filter
def truncatebefor(value, arg):
    """
    Truncates a string after a given number of chars
    Argument: Number of chars to truncate befor
    """
    try:
        length = int(arg)
    except ValueError:  # invalid literal for int()
        return value  # Fail silently.
    if not isinstance(value, basestring):
        value = str(value)
    if (len(value) > length):
        return "..."+value[len(value)-length:]
    else:
        return value


@register.filter
def shortening(value, arg):
    try:
        length = int(arg)
    except ValueError:  # invalid literal for int()
        return value  # Fail silently.
    front = length/2-3
    end = length/2-3

    if not isinstance(value, basestring):
        value = str(value)
    try:
        length = len(value)
        if length < length:
            return value
        elif length >= length:
            return value[:front]+"..."+value[length-end:]
    except Exception:
        logger.error(traceback.format_exc())
        return value


# See https://code.djangoproject.com/ticket/361
@register.filter
def subtract(value, arg):
    "Subtracts the arg from the value"
    return int(value) - int(arg)


# From http://djangosnippets.org/snippets/1357/
@register.filter
def get_range(value):
    """
    Filter - returns a list containing range made from given value
    Usage (in template):

    <ul>{% for i in 3|get_range %}
      <li>{{ i }}. Do something</li>
    {% endfor %}</ul>

    Results with the HTML:
    <ul>
      <li>0. Do something</li>
      <li>1. Do something</li>
      <li>2. Do something</li>
    </ul>

    Instead of 3 one may use the variable set in the views
    """
    return range(value)


@register.filter
def lengthformat(value):
    """
    Filter - returns the converted value
    all values are in micrometers
    """
    try:
        value = float(value)
    except (TypeError, ValueError, UnicodeDecodeError):
        return value

    if value < 0.001:
        return value * 1000 * 10
    elif value < 0.01:
        return value * 1000
    elif value < 1000:
        return value
    elif value < 1000 * 100:
        return value / 1000
    elif value < 1000 * 100 * 10:
        return value / 1000 / 100
    elif value < 1000 * 100 * 10 * 100:
        return value / 1000 / 100 / 10
    else:
        return value / 1000 / 100 / 10 / 1000


@register.filter
def lengthunit(value):
    """
    Filter - returns th emost suitable length units
    all values are in micrometers
    """

    if value == 0:
        return u'\u00B5m'
    elif value < 0.001:
        return u"\u212B"
    elif value < 0.01:
        return u"nm"
    elif value < 1000:
        return u'\u00B5m'
    elif value < 1000 * 100:
        return "mm"
    elif value < 1000 * 100 * 10:
        return "cm"
    elif value < 1000 * 100 * 10 * 100:
        return "m"
    else:
        return "km"


@register.filter
def timeformat(value):
    """
    Filter - returns the converted value with units
    all values are in seconds
    """
    from decimal import Decimal, InvalidOperation
    from django.utils.encoding import force_text

    if value is None:
        return ''
    try:
        value = Decimal(force_text(value))
    except UnicodeEncodeError:
        return u''
    except InvalidOperation:
        try:
            value = Decimal(force_text(float(value)))
        except (ValueError, InvalidOperation, TypeError, UnicodeEncodeError):
            return u'%s s' % str(value)
    # Formatting shows integer values for all, so we round() for accuracy
    if value == 0:
        return u'%d\u00A0s' % value
    if value < Decimal("0.001"):
        return u'%d\u00A0\u00B5s' % (round(value * 1000 * 1000))
    elif value < 1:
        return u'%d\u00A0ms' % (round(value * 1000))
    elif round(value) < 60:
        # Round and format seconds to one decimal place
        value = round(value * 10) / 10
        return u'%0.1f\u00A0s' % value
    elif round(value) < 60 * 60:
        value = round(value)        # Avoids '1min 60s'
        return u'%d\u00A0min\u00A0%d\u00A0s' % (value / 60, value % 60)
    else:
        value = round(value)        # Avoids '1h 60min'
        return u'%d\u00A0h\u00A0%d\u00A0min' % (value / 3600,
                                                round((value % 3600)/60))


# taken from https://code.djangoproject.com/ticket/17419
@register.filter
def json_dumps(value):
    return json.dumps(value)


# https://stackoverflow.com/questions/2415865/
# iterating-through-two-lists-in-django-templates
@register.filter(name='zip')
def zip_lists(a, b):
    return zip(a, b)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# #                Django settings for OMERO.web project.               # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#
# Copyright (c) 2008-2016 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2008.
#
# Version: 1.0
#


import os.path
import sys
import logging
import omero
import omero.config
import omero.clients
import tempfile
import re
import json
import random
import string
from builtins import str as text

from omero_ext import portalocker
from omero.util.concurrency import get_event
from omeroweb.utils import sort_properties_to_tuple
from omeroweb.connector import Server

logger = logging.getLogger(__name__)

# LOGS
# NEVER DEPLOY a site into production with DEBUG turned on.
# Debuging mode.
# A boolean that turns on/off debug mode.
# handler404 and handler500 works only when False
if 'OMERO_HOME' in os.environ:
    logger.warn("OMERO_HOME usage is ignored in OMERO.web")

OMERODIR = os.environ.get('OMERODIR')
if not OMERODIR:
    raise Exception('ERROR: OMERODIR not set')

# Logging
LOGDIR = os.path.join(OMERODIR, 'var', 'log').replace('\\', '/')

if not os.path.isdir(LOGDIR):
    try:
        os.makedirs(LOGDIR)
    except Exception:
        exctype, value = sys.exc_info()[:2]
        raise exctype(value)

# DEBUG: Never deploy a site into production with DEBUG turned on.
# Logging levels: logging.DEBUG, logging.INFO, logging.WARNING, logging.ERROR
# logging.CRITICAL
# FORMAT: 2010-01-01 00:00:00,000 INFO  [omeroweb.webadmin.webadmin_utils]
# (proc.1308 ) getGuestConnection:20 Open connection is not available

STANDARD_LOGFORMAT = (
    '%(asctime)s %(levelname)5.5s [%(name)40.40s]'
    ' (proc.%(process)5.5d) %(funcName)s():%(lineno)d %(message)s')

FULL_REQUEST_LOGFORMAT = (
    '%(asctime)s %(levelname)5.5s [%(name)40.40s]'
    ' (proc.%(process)5.5d) %(funcName)s():%(lineno)d'
    ' HTTP %(status_code)d %(request)s')

LOGGING_CLASS = 'omero_ext.cloghandler.ConcurrentRotatingFileHandler'
LOGSIZE = 500000000


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': STANDARD_LOGFORMAT
        },
        'full_request': {
            'format': FULL_REQUEST_LOGFORMAT
        },
    },
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse',
        },
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue',
        },
    },
    'handlers': {
        'default': {
            'level': 'DEBUG',
            'class': LOGGING_CLASS,
            'filename': os.path.join(
                LOGDIR, 'OMEROweb.log').replace('\\', '/'),
            'maxBytes': LOGSIZE,
            'backupCount': 10,
            'formatter': 'standard',
        },
        'request_handler': {
            'level': 'DEBUG',
            'class': LOGGING_CLASS,
            'filename': os.path.join(
                LOGDIR, 'OMEROweb.log').replace('\\', '/'),
            'maxBytes': LOGSIZE,
            'backupCount': 10,
            'filters': ['require_debug_false'],
            'formatter': 'full_request',
        },
        'console': {
            'level': 'INFO',
            'filters': ['require_debug_true'],
            'class': 'logging.StreamHandler',
            'formatter': 'standard'
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {  # Stop SQL debug from logging to main logger
            'handlers': ['default', 'request_handler', 'mail_admins'],
            'level': 'DEBUG',
            'propagate': False
        },
        'django': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True
        },
        '': {
            'handlers': ['default'],
            'level': 'DEBUG',
            'propagate': True
        }
    }
}


CONFIG_XML = os.path.join(OMERODIR, 'etc', 'grid', 'config.xml')
count = 10
event = get_event("websettings")

while True:
    try:
        CUSTOM_SETTINGS = dict()
        if os.path.exists(CONFIG_XML):
            CONFIG_XML = omero.config.ConfigXml(CONFIG_XML, read_only=True)
            CUSTOM_SETTINGS = CONFIG_XML.as_map()
            CONFIG_XML.close()
        break
    except portalocker.LockException:
        # logger.error("Exception while loading configuration retrying...",
        # exc_info=True)
        exctype, value = sys.exc_info()[:2]
        count -= 1
        if not count:
            raise exctype(value)
        else:
            event.wait(1)  # Wait a total of 10 seconds
    except Exception:
        # logger.error("Exception while loading configuration...",
        # exc_info=True)
        exctype, value = sys.exc_info()[:2]
        raise exctype(value)

del event
del count
del get_event

WSGI = "wsgi"
WSGITCP = "wsgi-tcp"
WSGI_TYPES = (WSGI, WSGITCP)
DEVELOPMENT = "development"
DEFAULT_SERVER_TYPE = WSGITCP
ALL_SERVER_TYPES = (WSGI, WSGITCP, DEVELOPMENT)

DEFAULT_SESSION_ENGINE = 'omeroweb.filesessionstore'
SESSION_ENGINE_VALUES = ('omeroweb.filesessionstore',
                         'django.contrib.sessions.backends.db',
                         'django.contrib.sessions.backends.file',
                         'django.contrib.sessions.backends.cache',
                         'django.contrib.sessions.backends.cached_db')


def parse_boolean(s):
    s = s.strip().lower()
    if s in ('true', '1', 't'):
        return True
    return False


def parse_paths(s):
    return [os.path.normpath(path) for path in json.loads(s)]


def check_server_type(s):
    if s not in ALL_SERVER_TYPES:
        raise ValueError(
            "Unknown server type: %s. Valid values are: %s"
            % (s, ALL_SERVER_TYPES))
    return s


def check_session_engine(s):
    if s not in SESSION_ENGINE_VALUES:
        raise ValueError(
            "Unknown session engine: %s. Valid values are: %s"
            % (s, SESSION_ENGINE_VALUES))
    return s


def identity(x):
    return x


def str_slash(s):
    if s is not None:
        s = str(s)
        if s and not s.endswith("/"):
            s += "/"
    return s


class LeaveUnset(Exception):
    pass


def leave_none_unset(s):
    if s is None:
        raise LeaveUnset()
    return s


def leave_none_unset_int(s):
    s = leave_none_unset(s)
    if s is not None:
        return int(s)


CUSTOM_HOST = CUSTOM_SETTINGS.get("Ice.Default.Host", "localhost")
CUSTOM_HOST = CUSTOM_SETTINGS.get("omero.master.host", CUSTOM_HOST)
# DO NOT EDIT!
INTERNAL_SETTINGS_MAPPING = {
    "omero.qa.feedback":
        ["FEEDBACK_URL", "http://qa.openmicroscopy.org.uk", str, None],
    "omero.web.upgrades.url":
        ["UPGRADES_URL", None, leave_none_unset, None],
    "omero.web.check_version":
        ["CHECK_VERSION", "true", parse_boolean, None],

    # Allowed hosts:
    # https://docs.djangoproject.com/en/1.8/ref/settings/#allowed-hosts
    "omero.web.allowed_hosts":
        ["ALLOWED_HOSTS", '["*"]', json.loads, None],

    # Do not show WARNING (1_8.W001): The standalone TEMPLATE_* settings
    # were deprecated in Django 1.8 and the TEMPLATES dictionary takes
    # precedence. You must put the values of the following settings
    # into your default TEMPLATES dict:
    # TEMPLATE_DIRS, TEMPLATE_CONTEXT_PROCESSORS.
    "omero.web.system_checks":
        ["SILENCED_SYSTEM_CHECKS", '["1_8.W001"]', json.loads, None],

    # Internal email notification for omero.web.admins,
    # loaded from config.xml directly
    "omero.mail.from":
        ["SERVER_EMAIL",
         None,
         identity,
         ("The email address that error messages come from, such as those"
          " sent to :property:`omero.web.admins`.  Requires EMAIL properties"
          " below.")],
    "omero.mail.host":
        ["EMAIL_HOST",
         None,
         identity,
         "The SMTP server host to use for sending email."],
    "omero.mail.password":
        ["EMAIL_HOST_PASSWORD",
         None,
         identity,
         "Password to use for the SMTP server."],
    "omero.mail.username":
        ["EMAIL_HOST_USER",
         None,
         identity,
         "Username to use for the SMTP server."],
    "omero.mail.port":
        ["EMAIL_PORT",
         25,
         identity,
         "Port to use for the SMTP server."],
    "omero.web.admins.email_subject_prefix":
        ["EMAIL_SUBJECT_PREFIX",
         "[OMERO.web - admin notification]",
         str,
         "Subject-line prefix for email messages"],
    "omero.mail.smtp.starttls.enable":
        ["EMAIL_USE_TLS",
         "false",
         parse_boolean,
         ("Whether to use a TLS (secure) connection when talking to the SMTP"
          " server.")],
}

CUSTOM_SETTINGS_MAPPINGS = {
    # Deployment configuration
    "omero.web.debug":
        ["DEBUG",
         "false",
         parse_boolean,
         ("A boolean that turns on/off debug mode. "
          "Use debug mode only in development, not in production, as it logs "
          "sensitive and confidential information in plaintext.")],
    "omero.web.secret_key":
        ["SECRET_KEY",
         None,
         leave_none_unset,
         ("A boolean that sets SECRET_KEY for a particular Django "
          "installation.")],
    "omero.web.admins":
        ["ADMINS",
         '[]',
         json.loads,
         ("A list of people who get code error notifications whenever the "
          "application identifies a broken link or raises an unhandled "
          "exception that results in an internal server error. This gives "
          "the administrators immediate notification of any errors, "
          "see :doc:`/sysadmins/mail`. "
          "Example:``'[[\"Full Name\", \"email address\"]]'``.")],
    "omero.web.application_server":
        ["APPLICATION_SERVER",
         DEFAULT_SERVER_TYPE,
         check_server_type,
         ("OMERO.web is configured to run in Gunicorn as a generic WSGI (TCP)"
          "application by default. Available options: ``wsgi-tcp`` "
          "(Gunicorn, default), ``wsgi`` (Advanced users only, e.g. manual "
          "Apache configuration with ``mod_wsgi``).")],
    "omero.web.application_server.host":
        ["APPLICATION_SERVER_HOST",
         "127.0.0.1",
         str,
         "Upstream application host"],
    "omero.web.application_server.port":
        ["APPLICATION_SERVER_PORT", 4080, int, "Upstream application port"],
    "omero.web.application_server.max_requests":
        ["APPLICATION_SERVER_MAX_REQUESTS", 0, int,
         ("The maximum number of requests a worker will process before "
          "restarting.")],
    "omero.web.middleware":
        ["MIDDLEWARE_CLASSES_LIST",
         ('['
          '{"index": 1, '
          '"class": "django.middleware.common.BrokenLinkEmailsMiddleware"},'
          '{"index": 2, '
          '"class": "django.middleware.common.CommonMiddleware"},'
          '{"index": 3, '
          '"class": "django.contrib.sessions.middleware.SessionMiddleware"},'
          '{"index": 4, '
          '"class": "django.middleware.csrf.CsrfViewMiddleware"},'
          '{"index": 5, '
          '"class": "django.contrib.messages.middleware.MessageMiddleware"},'
          '{"index": 6, '
          '"class": "django.middleware.clickjacking.XFrameOptionsMiddleware"}'
          ']'),
         json.loads,
         ('Warning: Only system administrators should use this feature. '
          'List of Django middleware classes in the form '
          '[{"class": "class.name", "index": FLOAT}]. '
          'See :djangodoc:`Django middleware <topics/http/middleware/>`.'
          ' Classes will be ordered by increasing index')],
    "omero.web.prefix":
        ["FORCE_SCRIPT_NAME",
         None,
         leave_none_unset,
         ("Used as the value of the SCRIPT_NAME environment variable in any"
          " HTTP request.")],
    "omero.web.use_x_forwarded_host":
        ["USE_X_FORWARDED_HOST",
         "false",
         parse_boolean,
         ("Specifies whether to use the X-Forwarded-Host header in preference "
          "to the Host header. This should only be enabled if a proxy which "
          "sets this header is in use.")],
    "omero.web.static_url":
        ["STATIC_URL",
         "/static/",
         str_slash,
         ("URL to use when referring to static files. Example: ``'/static/'``"
          " or ``'http://static.example.com/'``. Used as the base path for"
          " asset  definitions (the Media class) and the staticfiles app. It"
          " must end in a slash if set to a non-empty value.")],
    "omero.web.static_root":
        ["STATIC_ROOT",
         os.path.join(OMERODIR, 'var', 'static'),
         os.path.normpath,
         ("The absolute path to the directory where collectstatic will"
          " collect static files for deployment. If the staticfiles contrib"
          " app is enabled (default) the collectstatic management command"
          " will collect static files into this directory.")],
    "omero.web.session_engine":
        ["SESSION_ENGINE",
         DEFAULT_SESSION_ENGINE,
         check_session_engine,
         ("Controls where Django stores session data. See :djangodoc:"
          "`Configuring the session engine for more details <ref/settings"
          "/#session-engine>`.")],
    "omero.web.session_expire_at_browser_close":
        ["SESSION_EXPIRE_AT_BROWSER_CLOSE",
         "true",
         parse_boolean,
         ("A boolean that determines whether to expire the session when the "
          "user closes their browser. See :djangodoc:`Django Browser-length "
          "sessions vs. persistent sessions documentation <topics/http/"
          "sessions/#browser-length-vs-persistent-sessions>` for more "
          "details.")],
    "omero.web.caches":
        ["CACHES",
         ('{"default": {"BACKEND":'
          ' "django.core.cache.backends.dummy.DummyCache"}}'),
         json.loads,
         ("OMERO.web offers alternative session backends to automatically"
          " delete stale data using the cache session store backend, see "
          ":djangodoc:`Django cached session documentation <topics/http/"
          "sessions/#using-cached-sessions>` for more details.")],
    "omero.web.secure":
        ["SECURE",
         "false",
         parse_boolean,
         ("Force all backend OMERO.server connections to use SSL.")],
    "omero.web.session_cookie_age":
        ["SESSION_COOKIE_AGE",
         86400,
         int,
         "The age of session cookies, in seconds."],
    "omero.web.session_cookie_domain":
        ["SESSION_COOKIE_DOMAIN",
         None,
         leave_none_unset,
         "The domain to use for session cookies"],
    "omero.web.session_cookie_name":
        ["SESSION_COOKIE_NAME",
         None,
         leave_none_unset,
         "The name to use for session cookies"],
    "omero.web.session_cookie_secure":
        ["SESSION_COOKIE_SECURE",
         "false",
         parse_boolean,
         ("Restrict session cookies to HTTPS only, you are strongly "
          "recommended to set this to ``true`` in production.")],
    "omero.web.csrf_cookie_secure":
        ["CSRF_COOKIE_SECURE",
         "false",
         parse_boolean,
         ("Restrict CSRF cookies to HTTPS only, you are strongly "
          "recommended to set this to ``true`` in production.")],
    "omero.web.csrf_cookie_httponly":
        ["CSRF_COOKIE_HTTPONLY",
         "false",
         parse_boolean,
         ("Prevent CSRF cookie from being accessed in JavaScript. "
          "Currently disabled as it breaks background JavaScript POSTs in "
          "OMERO.web.")],

    "omero.web.logdir":
        ["LOGDIR", LOGDIR, str, "A path to the custom log directory."],
    "omero.web.secure_proxy_ssl_header":
        ["SECURE_PROXY_SSL_HEADER",
         '[]',
         json.loads,
         ("A tuple representing a HTTP header/value combination that "
          "signifies a request is secure. Example "
          "``'[\"HTTP_X_FORWARDED_PROTO_OMERO_WEB\", \"https\"]'``. "
          "For more details see :djangodoc:`secure proxy ssl header <ref/"
          "settings/#secure-proxy-ssl-header>`.")],
    "omero.web.wsgi_args":
        ["WSGI_ARGS",
         None,
         leave_none_unset,
         ("A string representing Gunicorn additional arguments. "
          "Check Gunicorn Documentation "
          "https://docs.gunicorn.org/en/latest/settings.html")],
    "omero.web.wsgi_workers":
        ["WSGI_WORKERS",
         5,
         int,
         ("The number of worker processes for handling requests. "
          "Check Gunicorn Documentation "
          "https://docs.gunicorn.org/en/stable/settings.html#workers")],
    "omero.web.wsgi_timeout":
        ["WSGI_TIMEOUT",
         60,
         int,
         ("Workers silent for more than this many seconds are killed "
          "and restarted. Check Gunicorn Documentation "
          "https://docs.gunicorn.org/en/stable/settings.html#timeout")],

    # Public user
    "omero.web.public.enabled":
        ["PUBLIC_ENABLED",
         "false",
         parse_boolean,
         "Enable and disable the OMERO.web public user functionality."],
    "omero.web.public.url_filter":
        ["PUBLIC_URL_FILTER",
         r'(?#This regular expression matches nothing)a^',
         re.compile,
         ("Set a regular expression that matches URLs the public user is "
          "allowed to access. If this is not set, no URLs will be "
          "publicly available.")],
    "omero.web.public.get_only":
        ["PUBLIC_GET_ONLY",
         "true",
         parse_boolean,
         "Restrict public users to GET requests only"],
    "omero.web.public.server_id":
        ["PUBLIC_SERVER_ID", 1, int, "Server to authenticate against."],
    "omero.web.public.user":
        ["PUBLIC_USER",
         None,
         leave_none_unset,
         "Username to use during authentication."],
    "omero.web.public.password":
        ["PUBLIC_PASSWORD",
         None,
         leave_none_unset,
         "Password to use during authentication."],
    "omero.web.public.cache.enabled":
        ["PUBLIC_CACHE_ENABLED", "false", parse_boolean, None],
    "omero.web.public.cache.key":
        ["PUBLIC_CACHE_KEY", "omero.web.public.cache.key", str, None],
    "omero.web.public.cache.timeout":
        ["PUBLIC_CACHE_TIMEOUT", 60 * 60 * 24, int, None],

    # Social media integration
    "omero.web.sharing.twitter":
        ["SHARING_TWITTER",
         '{}',
         json.loads,
         ("Dictionary of `server-name: @twitter-site-username`, where "
          "server-name matches a name from `omero.web.server_list`. "
          "For example: ``'{\"omero\": \"@openmicroscopy\"}'``")],
    "omero.web.sharing.opengraph":
        ["SHARING_OPENGRAPH",
         '{}',
         json.loads,
         ("Dictionary of `server-name: site-name`, where "
          "server-name matches a name from `omero.web.server_list`. "
          "For example: ``'{\"omero\": \"Open Microscopy\"}'``")],

    # Application configuration
    "omero.web.server_list":
        ["SERVER_LIST",
         '[["%s", 4064, "omero"]]' % CUSTOM_HOST,
         json.loads,
         "A list of servers the Web client can connect to."],
    "omero.web.ping_interval":
        ["PING_INTERVAL",
         60000,
         int,
         "Timeout interval between ping invocations in seconds"],
    "omero.web.chunk_size":
        ["CHUNK_SIZE",
         1048576,
         int,
         "Size, in bytes, of the “chunk”"],
    "omero.web.webgateway_cache":
        ["WEBGATEWAY_CACHE", None, leave_none_unset, None],

    # VIEWER
    "omero.web.viewer.view":
        ["VIEWER_VIEW",
         'omeroweb.webclient.views.image_viewer',
         str,
         ("Django view which handles display of, or redirection to, the "
          "desired full image viewer.")],

    # OPEN WITH
    "omero.web.open_with":
        ["OPEN_WITH",
         ('[["Image viewer", "webgateway", {"supported_objects": ["image"],'
          '"script_url": "webclient/javascript/ome.openwith_viewer.js"}]]'),
         json.loads,
         ("A list of viewers that can be used to display selected Images "
          "or other objects. Each viewer is defined as "
          "``[\"Name\", \"url\", options]``. Url is reverse(url). "
          "Selected objects are added to the url as ?image=:1&image=2"
          "Objects supported must be specified in options with "
          "e.g. ``{\"supported_objects\":[\"images\"]}`` "
          "to enable viewer for one or more images.")],

    # PIPELINE 1.3.20

    # Pipeline is an asset packaging library for Django, providing both CSS
    # and JavaScript concatenation and compression, built-in JavaScript
    # template support, and optional data-URI image and font embedding.
    "omero.web.pipeline_js_compressor":
        ["PIPELINE_JS_COMPRESSOR",
         None,
         identity,
         ("Compressor class to be applied to JavaScript files. If empty or "
          "None, JavaScript files won't be compressed.")],
    "omero.web.pipeline_css_compressor":
        ["PIPELINE_CSS_COMPRESSOR",
         None,
         identity,
         ("Compressor class to be applied to CSS files. If empty or None,"
          " CSS files won't be compressed.")],
    "omero.web.pipeline_staticfile_storage":
        ["STATICFILES_STORAGE",
         "pipeline.storage.PipelineStorage",
         str,
         ("The file storage engine to use when collecting static files with"
          " the collectstatic management command. See `the documentation "
          "<https://django-pipeline.readthedocs.org/en/latest/storages.html>`_"
          " for more details.")],

    # Customisation
    "omero.web.login_logo":
        ["LOGIN_LOGO",
         None,
         leave_none_unset,
         ("Customize webclient login page with your own logo. Logo images "
          "should ideally be 150 pixels high or less and will appear above "
          "the OMERO logo. You will need to host the image somewhere else "
          "and link to it with the OMERO logo.")],
    "omero.web.login_view":
        ["LOGIN_VIEW",
         "weblogin",
         str,
         ("The Django view name used for login. Use this to provide an "
          "alternative login workflow.")],
    "omero.web.login_incorrect_credentials_text":
        ["LOGIN_INCORRECT_CREDENTIALS_TEXT",
         "Connection not available, please check your user name and password.",
         str,
         ("The error message shown to users who enter an incorrect username "
          "or password.")],

    "omero.web.top_logo":
        ["TOP_LOGO",
         "",
         str,
         ("Customize the webclient top bar logo. The recommended image height "
          "is 23 pixels and it must be hosted outside of OMERO.web.")],
    "omero.web.top_logo_link":
        ["TOP_LOGO_LINK",
         "",
         str,
         ("The target location of the webclient top logo, default unlinked.")],

    "omero.web.user_dropdown":
        ["USER_DROPDOWN",
         "true",
         parse_boolean,
         ("Whether or not to include a user dropdown in the base template."
          " Particularly useful when used in combination with the OMERO.web"
          " public user where logging in may not make sense.")],
    "omero.web.feedback.comment.enabled":
        ["FEEDBACK_COMMENT_ENABLED",
         "true",
         parse_boolean,
         ("Enable the feedback form for comments. "
          "These comments are sent to the URL in ``omero.qa.feedback`` "
          "(OME team by default).")],
    "omero.web.feedback.error.enabled":
        ["FEEDBACK_ERROR_ENABLED",
         "true",
         parse_boolean,
         ("Enable the feedback form for errors. "
          "These errors are sent to the URL in ``omero.qa.feedback`` "
          "(OME team by default).")],
    "omero.web.staticfile_dirs":
        ["STATICFILES_DIRS",
         '[]',
         json.loads,
         ("Defines the additional locations the staticfiles app will traverse"
          " if the FileSystemFinder finder is enabled, e.g. if you use the"
          " collectstatic or findstatic management command or use the static"
          " file serving view.")],
    "omero.web.template_dirs":
        ["TEMPLATE_DIRS",
         '[]',
         json.loads,
         ("List of locations of the template source files, in search order. "
          "Note that these paths should use Unix-style forward slashes.")],
    "omero.web.index_template":
        ["INDEX_TEMPLATE",
         None,
         identity,
         ("Define template used as an index page ``http://your_host/omero/``."
          "If None user is automatically redirected to the login page."
          "For example use 'webclient/index.html'. ")],
    "omero.web.base_include_template":
        ["BASE_INCLUDE_TEMPLATE",
         None,
         identity,
         ("Template to be included in every page, at the end of the <body>")],
    "omero.web.login_redirect":
        ["LOGIN_REDIRECT",
         '{}',
         json.loads,
         ("Redirect to the given location after logging in. It only supports "
          "arguments for :djangodoc:`Django reverse function"
          " <ref/urlresolvers/#reverse>`. "
          "For example: ``'{\"redirect\": [\"webindex\"], \"viewname\":"
          " \"load_template\", \"args\":[\"userdata\"], \"query_string\":"
          " {\"experimenter\": -1}}'``")],

    "omero.web.login.show_client_downloads":
        ["SHOW_CLIENT_DOWNLOADS",
         "true",
         parse_boolean,
         ("Whether to link to official client downloads on the login page")],
    "omero.web.login.client_downloads_base":
        ["CLIENT_DOWNLOAD_GITHUB_REPO",
         'ome/omero-insight',
         str,
         ("GitHub repository containing the Desktop client downloads")],

    "omero.web.apps":
        ["ADDITIONAL_APPS",
         '[]',
         json.loads,
         ("Add additional Django applications. For example, see"
          " :doc:`/developers/Web/CreateApp`")],
    "omero.web.databases":
        ["DATABASES", '{}', json.loads, None],
    "omero.web.page_size":
        ["PAGE",
         200,
         int,
         ("Number of images displayed within a dataset or 'orphaned'"
          " container to prevent from loading them all at once.")],
    "omero.web.thumbnails_batch":
        ["THUMBNAILS_BATCH",
         50,
         int,
         ("Number of thumbnails retrieved to prevent from loading them"
          " all at once. Make sure the size is not too big, otherwise"
          " you may exceed limit request line, see"
          " https://docs.gunicorn.org/en/latest/settings.html"
          "?highlight=limit_request_line")],
    "omero.web.ui.top_links":
        ["TOP_LINKS",
         ('['
          '["Data", "webindex", {"title": "Browse Data via Projects, Tags'
          ' etc"}],'
          '["History", "history", {"title": "History"}],'
          '["Help", "https://help.openmicroscopy.org/",'
          '{"title":"Open OMERO user guide in a new tab", "target":"new"}]'
          ']'),
         json.loads,
         ("Add links to the top header: links are ``['Link Text', "
          "'link|lookup_view', options]``, where the url is reverse('link'), "
          "simply 'link' (for external urls) or lookup_view is a detailed "
          "dictionary {\"viewname\": \"str\", \"args\": [], \"query_string\": "
          "{\"param\": \"value\" }], "
          "E.g. ``'[\"Webtest\", \"webtest_index\"] or [\"Homepage\","
          " \"http://...\", {\"title\": \"Homepage\", \"target\": \"new\"}"
          " ] or [\"Repository\", {\"viewname\": \"webindex\", "
          "\"query_string\": {\"experimenter\": -1}}, "
          "{\"title\": \"Repo\"}]'``")],
    "omero.web.ui.metadata_panes":
        ["METADATA_PANES",
         ('['
          '{"name": "tag", "label": "Tags", "index": 1},'
          '{"name": "map", "label": "Key-Value Pairs", "index": 2},'
          '{"name": "table", "label": "Tables", "index": 3},'
          '{"name": "file", "label": "Attachments", "index": 4},'
          '{"name": "comment", "label": "Comments", "index": 5},'
          '{"name": "rating", "label": "Ratings", "index": 6},'
          '{"name": "other", "label": "Others", "index": 7}'
          ']'),
         json.loads,
         ("Manage Metadata pane accordion. This functionality is limited to"
          " the existing sections.")],
    "omero.web.ui.right_plugins":
        ["RIGHT_PLUGINS",
         ('[["Acquisition",'
          ' "webclient/data/includes/right_plugin.acquisition.js.html",'
          ' "metadata_tab"],'
          # '["ROIs", "webtest/webclient_plugins/right_plugin.rois.js.html",
          # "image_roi_tab"],'
          '["Preview", "webclient/data/includes/right_plugin.preview.js.html"'
          ', "preview_tab"]]'),
         json.loads,
         ("Add plugins to the right-hand panel. "
          "Plugins are ``['Label', 'include.js', 'div_id']``. "
          "The javascript loads data into ``$('#div_id')``.")],
    "omero.web.ui.center_plugins":
        ["CENTER_PLUGINS",
         ('['
          # '["Split View",
          # "webtest/webclient_plugins/center_plugin.splitview.js.html",
          # "split_view_panel"],'
          ']'),
         json.loads,
         ("Add plugins to the center panels. Plugins are "
          "``['Channel overlay',"
          " 'webtest/webclient_plugins/center_plugin.overlay.js.html',"
          " 'channel_overlay_panel']``. "
          "The javascript loads data into ``$('#div_id')``.")],

    # CORS
    "omero.web.cors_origin_whitelist":
        ["CORS_ORIGIN_WHITELIST",
         '[]',
         json.loads,
         ("A list of origin hostnames that are authorized to make cross-site "
          "HTTP requests. "
          "Used by the django-cors-headers app as described at "
          "https://github.com/ottoyiu/django-cors-headers")],
    "omero.web.cors_origin_allow_all":
        ["CORS_ORIGIN_ALLOW_ALL",
         "false",
         parse_boolean,
         ("If True, cors_origin_whitelist will not be used and all origins "
          "will be authorized to make cross-site HTTP requests.")],

    "omero.web.x_frame_options":
        ["X_FRAME_OPTIONS",
         "SAMEORIGIN",
         str,
         "Whether to allow OMERO.web to be loaded in a frame."
         ],

    "omero.web.django_additional_settings":
        ["DJANGO_ADDITIONAL_SETTINGS",
         "[]",
         json.loads,
         ("Additional Django settings as list of key-value tuples. "
          "Use this to set or override Django settings that aren't managed by "
          "OMERO.web. E.g. ``[\"CUSTOM_KEY\", \"CUSTOM_VALUE\"]``")],
    "omero.web.nginx_server_extra_config":
        ["NGINX_SERVER_EXTRA_CONFIG",
         "[]",
         json.loads,
         ("Extra configuration lines to add to the Nginx server block. "
          "Lines will be joined with \\n. "
          "Remember to terminate lines with; when necessary.")],
}

DEPRECATED_SETTINGS_MAPPINGS = {
    # Deprecated settings, description should indicate the replacement.
    "omero.web.force_script_name":
        ["FORCE_SCRIPT_NAME",
         None,
         leave_none_unset,
         ("Use omero.web.prefix instead.")],
    "omero.web.server_email":
        ["SERVER_EMAIL",
         None,
         identity,
         ("Use omero.mail.from instead.")],
    "omero.web.email_host":
        ["EMAIL_HOST",
         None,
         identity,
         ("Use omero.mail.host instead.")],
    "omero.web.email_host_password":
        ["EMAIL_HOST_PASSWORD",
         None,
         identity,
         ("Use omero.mail.password instead.")],
    "omero.web.email_host_user":
        ["EMAIL_HOST_USER",
         None,
         identity,
         ("Use omero.mail.username instead.")],
    "omero.web.email_port":
        ["EMAIL_PORT",
         None,
         identity,
         ("Use omero.mail.port instead.")],
    "omero.web.email_subject_prefix":
        ["EMAIL_SUBJECT_PREFIX",
         "[OMERO.web]",
         str,
         ("Default email subject is no longer configurable.")],
    "omero.web.email_use_tls":
        ["EMAIL_USE_TLS",
         "false",
         parse_boolean,
         ("Use omero.mail.smtp.* instead to set up"
          " javax.mail.Session properties.")],
    "omero.web.plate_download.enabled":
        ["PLATE_DOWNLOAD_ENABLED",
         "false",
         parse_boolean,
         ("Use omero.policy.binary_access instead to restrict download.")],
    "omero.web.viewer.initial_zoom_level":
        ["VIEWER_INITIAL_ZOOM_LEVEL",
         None,
         leave_none_unset_int,
         ("Use omero.client.viewer.initial_zoom_level instead.")],
    "omero.web.send_broken_link_emails":
        ["SEND_BROKEN_LINK_EMAILS",
         "false",
         parse_boolean,
         ("Replaced by django.middleware.common.BrokenLinkEmailsMiddleware."
          "To get notification set :property:`omero.web.admins` property.")
         ],
}

del CUSTOM_HOST


def check_worker_class(c):
    if c == "gevent":
        try:
            import gevent  # NOQA
        except ImportError:
            raise ImportError("You are using async workers based "
                              "on Greenlets via Gevent. Install gevent")
    return str(c)


def check_threading(t):
    if t > 1:
        try:
            import concurrent.futures  # NOQA
        except ImportError:
            raise ImportError("You are using sync workers with "
                              "multiple threads. Install futures")
    return int(t)


# DEVELOPMENT_SETTINGS_MAPPINGS - WARNING: For each setting developer MUST open
# a ticket that needs to be resolved before a release either by moving the
# setting to CUSTOM_SETTINGS_MAPPINGS or by removing the setting at all.
DEVELOPMENT_SETTINGS_MAPPINGS = {
    "omero.web.wsgi_worker_class":
        ["WSGI_WORKER_CLASS",
         "sync",
         check_worker_class,
         ("The default OMERO.web uses sync workers to handle most “normal” "
          "types of workloads. Check Gunicorn Design Documentation "
          "https://docs.gunicorn.org/en/stable/design.html")],
    "omero.web.wsgi_worker_connections":
        ["WSGI_WORKER_CONNECTIONS",
         1000,
         int,
         ("(ASYNC WORKERS only) The maximum number of simultaneous clients. "
          "Check Gunicorn Documentation https://docs.gunicorn.org"
          "/en/stable/settings.html#worker-connections")],
    "omero.web.wsgi_threads":
        ["WSGI_THREADS",
         1,
         check_threading,
         ("(SYNC WORKERS only) The number of worker threads for handling "
          "requests. Check Gunicorn Documentation "
          "https://docs.gunicorn.org/en/stable/settings.html#threads")],
}


def map_deprecated_settings(settings):
    m = {}
    for key, values in settings.items():
        try:
            global_name = values[0]
            m[global_name] = (CUSTOM_SETTINGS[key], key)
            if len(values) < 5:
                # Not using default (see process_custom_settings)
                values.append(False)
        except KeyError:
            if len(values) < 5:
                values.append(True)
    return m


def process_custom_settings(
        module, settings='CUSTOM_SETTINGS_MAPPINGS', deprecated=None):
    logging.info('Processing custom settings for module %s' % module.__name__)

    if deprecated:
        deprecated_map = map_deprecated_settings(
            getattr(module, deprecated, {}))
    else:
        deprecated_map = {}

    for key, values in getattr(module, settings, {}).items():
        # Django may import settings.py more than once, see:
        # http://blog.dscpl.com.au/2010/03/improved-wsgi-script-for-use-with.html
        # In that case, the custom settings have already been processed.
        if len(values) == 5:
            continue

        global_name, default_value, mapping, description = values

        try:
            global_value = CUSTOM_SETTINGS[key]
            values.append(False)
        except KeyError:
            global_value = default_value
            values.append(True)

        try:
            using_default = values[-1]
            if global_name in deprecated_map:
                dep_value, dep_key = deprecated_map[global_name]
                if using_default:
                    logging.warning(
                        'Setting %s is deprecated, use %s', dep_key, key)
                    global_value = dep_value
                else:
                    logging.error(
                        '%s and its deprecated key %s are both set, using %s',
                        key, dep_key, key)
            setattr(module, global_name, mapping(global_value))
        except ValueError as e:
            raise ValueError(
                "Invalid %s (%s = %r). %s. %s" %
                (global_name, key, global_value, e.args[0], description))
        except ImportError as e:
            raise ImportError(
                "ImportError: %s. %s (%s = %r).\n%s" %
                (e.message, global_name, key, global_value, description))
        except LeaveUnset:
            pass


process_custom_settings(sys.modules[__name__], 'INTERNAL_SETTINGS_MAPPING')
process_custom_settings(sys.modules[__name__], 'CUSTOM_SETTINGS_MAPPINGS',
                        'DEPRECATED_SETTINGS_MAPPINGS')
process_custom_settings(sys.modules[__name__], 'DEVELOPMENT_SETTINGS_MAPPINGS')

if not DEBUG:  # from CUSTOM_SETTINGS_MAPPINGS  # noqa
    LOGGING['loggers']['django.request']['level'] = 'INFO'
    LOGGING['loggers']['django']['level'] = 'INFO'
    LOGGING['loggers']['']['level'] = 'INFO'


def report_settings(module):
    from django.views.debug import cleanse_setting
    custom_settings_mappings = getattr(module, 'CUSTOM_SETTINGS_MAPPINGS', {})
    for key in sorted(custom_settings_mappings):
        values = custom_settings_mappings[key]
        global_name, default_value, mapping, description, using_default = \
            values
        source = using_default and "default" or key
        global_value = getattr(module, global_name, None)
        if global_name.isupper():
            logger.debug(
                "%s = %r (source:%s)", global_name,
                cleanse_setting(global_name, global_value), source)

    deprecated_settings = getattr(module, 'DEPRECATED_SETTINGS_MAPPINGS', {})
    for key in sorted(deprecated_settings):
        values = deprecated_settings[key]
        global_name, default_value, mapping, description, using_default = \
            values
        global_value = getattr(module, global_name, None)
        if global_name.isupper() and not using_default:
            logger.debug(
                "%s = %r (deprecated:%s, %s)", global_name,
                cleanse_setting(global_name, global_value), key, description)


report_settings(sys.modules[__name__])

SITE_ID = 1

# Local time zone for this installation. Choices can be found here:
# http://www.postgresql.org/docs/8.1/static/datetime-keywords.html#DATETIME-TIMEZONE-SET-TABLE
# although not all variations may be possible on all operating systems.
TIME_ZONE = 'Europe/London'
FIRST_DAY_OF_WEEK = 0     # 0-Monday, ... 6-Sunday

# LANGUAGE_CODE: A string representing the language code for this
# installation. This should be in standard language format. For example, U.S.
# English is "en-us".
LANGUAGE_CODE = 'en-gb'

# SECRET_KEY: A secret key for this particular Django installation. Used to
# provide a seed in secret-key hashing algorithms. Set this to a random string,
# the longer, the better. Make this unique, and don't share it with anybody.
try:
    SECRET_KEY
except NameError:
    secret_path = os.path.join(OMERODIR, 'var',
                               'django_secret_key').replace('\\', '/')
    if not os.path.isfile(secret_path):
        try:
            secret_key = ''.join(
                [random.SystemRandom()
                 .choice("{0}{1}{2}"
                 .format(string.ascii_letters,
                         string.digits,
                         string.punctuation)) for i in range(50)]
            )
            with os.fdopen(os.open(secret_path,
                                   os.O_WRONLY | os.O_CREAT,
                                   0o600), 'w') as secret_file:
                secret_file.write(secret_key)
        except IOError:
            raise IOError("Please create a %s file with random characters"
                          " to generate your secret key!" % secret_path)
    try:
        with open(secret_path, 'r') as secret_file:
            SECRET_KEY = secret_file.read().strip()
    except IOError:
        raise IOError("Could not find secret key in %s!" % secret_path)

# USE_I18N: A boolean that specifies whether Django's internationalization
# system should be enabled.
# This provides an easy way to turn it off, for performance. If this is set to
# False, Django will make some optimizations so as not to load the
# internationalization machinery.
USE_I18N = True

# ROOT_URLCONF: A string representing the full Python import path to your root
# URLconf.
# For example: "mydjangoapps.urls". Can be overridden on a per-request basis
# by setting the attribute urlconf on the incoming HttpRequest object.
ROOT_URLCONF = 'omeroweb.urls'

# STATICFILES_FINDERS: The list of finder backends that know how to find
# static files in various locations. The default will find files stored in the
# STATICFILES_DIRS setting (using
# django.contrib.staticfiles.finders.FileSystemFinder) and in a static
# subdirectory of each app (using
# django.contrib.staticfiles.finders.AppDirectoriesFinder)
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "pipeline.finders.PipelineFinder",
)

# STATICFILES_DIRS: This setting defines the additional locations the
# staticfiles app will traverse if the FileSystemFinder finder is enabled,
# e.g. if you use the collectstatic or findstatic management command or use
# the static file serving view.
# from CUSTOM_SETTINGS_MAPPINGS
# STATICFILES_DIRS += (("webapp/custom_static", path/to/statics),)  # noqa

# TEMPLATES: A list containing the settings for all template engines
# to be used with Django. Each item of the list is a dictionary containing
# the options for an individual engine.
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': TEMPLATE_DIRS,  # noqa
        'APP_DIRS': True,
        'OPTIONS': {
            'builtins': ['omeroweb.webgateway.templatetags.defaulttags'],
            'debug': DEBUG,  # noqa
            'context_processors': [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven't customized them:
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
                'omeroweb.custom_context_processor.url_suffix',
                'omeroweb.custom_context_processor.base_include_template',
            ],
        },
    },
]

# INSTALLED_APPS: A tuple of strings designating all applications that are
# enabled in this Django installation. Each string should be a full Python
# path to a Python package that contains a Django application, as created by
# django-admin.py startapp.
INSTALLED_APPS = (
    'django.contrib.staticfiles',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
)

# ADDITONAL_APPS: We import any settings.py from apps. This allows them to
# modify settings.
# We're also processing any CUSTOM_SETTINGS_MAPPINGS defined there.
for app in ADDITIONAL_APPS:  # from CUSTOM_SETTINGS_MAPPINGS  # noqa
    # Previously the app was added to INSTALLED_APPS as 'omeroweb.app', which
    # then required the app to reside within or be symlinked from within
    # omeroweb, instead of just having to be somewhere on the python path.
    # To allow apps to just be on the path, but keep it backwards compatible,
    # try to import as omeroweb.app, if it works, keep that in INSTALLED_APPS,
    # otherwise add it to INSTALLED_APPS just with its own name.
    try:
        __import__('omeroweb.%s' % app)
        INSTALLED_APPS += ('omeroweb.%s' % app,)
    except ImportError:
        INSTALLED_APPS += (app,)
    try:
        logger.debug(
            'Attempting to import additional app settings for app: %s' % app)
        module = __import__('%s.settings' % app)
        process_custom_settings(module.settings)
        report_settings(module.settings)
    except ImportError:
        logger.debug("Couldn't import settings from app: %s" % app)

INSTALLED_APPS += (
    'omeroweb.feedback',
    'omeroweb.webadmin',
    'omeroweb.webclient',
    'omeroweb.webgateway',
    'omeroweb.webredirect',
    'omeroweb.api',
    'pipeline',
)

logger.debug('INSTALLED_APPS=%s' % [INSTALLED_APPS])


PIPELINE = {
    'STYLESHEETS': {
        'webgateway_viewer': {
            'source_filenames': (
                'webgateway/css/reset.css',
                'webgateway/css/ome.body.css',
                'webclient/css/dusty.css',
                'webgateway/css/ome.viewport.css',
                'webgateway/css/ome.toolbar.css',
                'webgateway/css/ome.gs_slider.css',
                'webgateway/css/base.css',
                'webgateway/css/ome.snippet_header_logo.css',
                'webgateway/css/ome.postit.css',
                '3rdparty/farbtastic-1.2/farbtastic.css',
                'webgateway/css/ome.colorbtn.css',
                '3rdparty/JQuerySpinBtn-1.3a/JQuerySpinBtn.css',
                '3rdparty/jquery-ui-1.10.4/themes/base/jquery-ui.all.css',
                'webgateway/css/omero_image.css',
                '3rdparty/panojs-2.0.0/panojs.css',
            ),
            'output_filename': 'omeroweb.viewer.min.css',
        },
    },
    'CSS_COMPRESSOR': 'pipeline.compressors.NoopCompressor',
    'JS_COMPRESSOR': 'pipeline.compressors.NoopCompressor',
    'JAVASCRIPT': {
        'webgateway_viewer': {
            'source_filenames': (
                '3rdparty/jquery-1.11.1.js',
                '3rdparty/jquery-migrate-1.2.1.js',
                '3rdparty/jquery-ui-1.10.4/js/jquery-ui.1.10.4.js',
                'webgateway/js/ome.popup.js',
                '3rdparty/aop-1.3.js',
                '3rdparty/raphael-2.1.0/raphael.js',
                '3rdparty/raphael-2.1.0/scale.raphael.js',
                '3rdparty/panojs-2.0.0/utils.js',
                '3rdparty/panojs-2.0.0/PanoJS.js',
                '3rdparty/panojs-2.0.0/controls.js',
                '3rdparty/panojs-2.0.0/pyramid_Bisque.js',
                '3rdparty/panojs-2.0.0/pyramid_imgcnv.js',
                '3rdparty/panojs-2.0.0/pyramid_Zoomify.js',
                '3rdparty/panojs-2.0.0/control_thumbnail.js',
                '3rdparty/panojs-2.0.0/control_info.js',
                '3rdparty/panojs-2.0.0/control_svg.js',
                '3rdparty/panojs-2.0.0/control_roi.js',
                '3rdparty/panojs-2.0.0/control_scalebar.js',
                '3rdparty/hammer-2.0.2/hammer.min.js',
                'webgateway/js/ome.gs_utils.js',
                'webgateway/js/ome.viewportImage.js',
                'webgateway/js/ome.gs_slider.js',
                'webgateway/js/ome.viewport.js',
                'webgateway/js/omero_image.js',
                'webgateway/js/ome.roidisplay.js',
                'webgateway/js/ome.scalebardisplay.js',
                'webgateway/js/ome.smartdialog.js',
                'webgateway/js/ome.roiutils.js',
                '3rdparty/JQuerySpinBtn-1.3a/JQuerySpinBtn.js',
                'webgateway/js/ome.colorbtn.js',
                'webgateway/js/ome.postit.js',
                '3rdparty/jquery.selectboxes-2.2.6.js',
                '3rdparty/farbtastic-1.2/farbtastic.js',
                '3rdparty/jquery.mousewheel-3.0.6.js',
            ),
            'output_filename': 'omeroweb.viewer.min.js',
        }
    }
}

# Prevent scripting attacks from obtaining session cookie
SESSION_COOKIE_HTTPONLY = True

CSRF_FAILURE_VIEW = "omeroweb.feedback.views.csrf_failure"

# Configuration for django-cors-headers app
# See https://github.com/ottoyiu/django-cors-headers
# Configration of allowed origins is handled by custom settings above
CORS_ALLOW_CREDENTIALS = True
# Needed for Django <1.9 since CSRF_TRUSTED_ORIGINS not supported
CORS_REPLACE_HTTPS_REFERER = True

# FEEDBACK - DO NOT MODIFY!
# FEEDBACK_URL: Is now configurable for testing purpuse only. Used in
# feedback.sendfeedback.SendFeedback class in order to submit errors or
# comment messages to http://qa.openmicroscopy.org.uk.
# FEEDBACK_APP: 6 = OMERO.web
FEEDBACK_APP = 6

# IGNORABLE_404_STARTS:
# Default: ('/cgi-bin/', '/_vti_bin', '/_vti_inf')
# IGNORABLE_404_ENDS:
# Default: ('mail.pl', 'mailform.pl', 'mail.cgi', 'mailform.cgi',
# 'favicon.ico', '.php')

# SESSION_FILE_PATH: If you're using file-based session storage, this sets the
# directory in which Django will store session data. When the default value
# (None) is used, Django will use the standard temporary directory for the
# system.
SESSION_FILE_PATH = tempfile.gettempdir()

# FILE_UPLOAD_TEMP_DIR: The directory to store data temporarily while
# uploading files.
FILE_UPLOAD_TEMP_DIR = tempfile.gettempdir()

# # FILE_UPLOAD_MAX_MEMORY_SIZE: The maximum size (in bytes) that an upload
# will be before it gets streamed to the file system.
FILE_UPLOAD_MAX_MEMORY_SIZE = 2621440  # default 2621440 (i.e. 2.5 MB).

# DEFAULT_IMG: Used in
# webclient.webclient_gateway.OmeroWebGateway.defaultThumbnail in order to
# load default image while thumbnail can't be retrieved from the server.
DEFAULT_IMG = os.path.join(
    os.path.dirname(__file__), 'webgateway', 'static', 'webgateway', 'img',
    'image128.png').replace('\\', '/')

# # DEFAULT_USER: Used in
# webclient.webclient_gateway.OmeroWebGateway.getExperimenterDefaultPhoto in
# order to load default avatar while experimenter photo can't be retrieved
# from the server.
DEFAULT_USER = os.path.join(
    os.path.dirname(__file__), 'webgateway', 'static', 'webgateway', 'img',
    'personal32.png').replace('\\', '/')

# MANAGERS: A tuple in the same format as ADMINS that specifies who should get
# broken-link notifications when
# SEND_BROKEN_LINK_EMAILS=True.
MANAGERS = ADMINS  # from CUSTOM_SETTINGS_MAPPINGS  # noqa

# https://docs.djangoproject.com/en/1.6/releases/1.6/#default-session-serialization-switched-to-json
# JSON serializer, which is now the default, cannot handle
# omeroweb.connector.Connector object
SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'

# Load custom settings from etc/grid/config.xml
# Tue  2 Nov 2010 11:03:18 GMT -- ticket:3228
# MIDDLEWARE: A tuple of middleware classes to use.
MIDDLEWARE = sort_properties_to_tuple(MIDDLEWARE_CLASSES_LIST)  # noqa

for k, v in DJANGO_ADDITIONAL_SETTINGS:  # noqa
    setattr(sys.modules[__name__], k, v)


# Load server list and freeze
def load_server_list():
    for s in SERVER_LIST:  # from CUSTOM_SETTINGS_MAPPINGS  # noqa
        server = (len(s) > 2) and text(s[2]) or None
        Server(host=text(s[0]), port=int(s[1]), server=server)
    Server.freeze()


load_server_list()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
#
# Copyright (c) 2008-2014 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2008.
#
# Version: 1.0
#

import logging
import pkgutil
from django.conf import settings
from django.apps import AppConfig
from django.conf.urls import url, include
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.shortcuts import redirect

from django.core.urlresolvers import reverse
from django.utils.functional import lazy
from django.views.generic import RedirectView
from django.views.decorators.cache import never_cache
from omeroweb.webclient import views as webclient_views

logger = logging.getLogger(__name__)

# error handler
handler404 = "omeroweb.feedback.views.handler404"
handler500 = "omeroweb.feedback.views.handler500"

reverse_lazy = lazy(reverse, str)


def redirect_urlpatterns():
    """
    Helper function to return a URL pattern for index page http://host/.
    """
    if settings.INDEX_TEMPLATE is None:
        return [
            url(r'^$', never_cache(
                RedirectView.as_view(url=reverse_lazy('webindex'),
                                     permanent=True)),
                name="index")
        ]
    else:
        return [
            url(r'^$', never_cache(
                RedirectView.as_view(url=reverse_lazy('webindex_custom'),
                                     permanent=True)),
                name="index"),
        ]


# url patterns

urlpatterns = []

for app in settings.ADDITIONAL_APPS:
    if isinstance(app, AppConfig):
        app_config = app
    else:
        app_config = AppConfig.create(app)
    label = app_config.label

    # Depending on how we added the app to INSTALLED_APPS in settings.py,
    # include the urls the same way
    if 'omeroweb.%s' % app in settings.INSTALLED_APPS:
        urlmodule = 'omeroweb.%s.urls' % app
    else:
        urlmodule = '%s.urls' % app

    # Try to import module.urls.py if it exists (not for corsheaders etc)
    urls_found = pkgutil.find_loader(urlmodule)
    if urls_found is not None:
        try:
            __import__(urlmodule)
            regex = '^(?i)%s/' % label
            urlpatterns.append(url(regex, include(urlmodule)))
        except ImportError:
            print("""Failed to import %s
Please check if the app is installed and the versions of the app and
OMERO.web are compatible
            """ % urlmodule)
            raise
    else:
        logger.debug('Module not found: %s' % urlmodule)

urlpatterns += [
    url(r'^favicon\.ico$',
        lambda request: redirect('%swebgateway/img/ome.ico'
                                 % settings.STATIC_URL)),
    url(r'^(?i)webgateway/', include('omeroweb.webgateway.urls')),
    url(r'^(?i)webadmin/', include('omeroweb.webadmin.urls')),
    url(r'^(?i)webclient/', include('omeroweb.webclient.urls')),

    url(r'^(?i)url/', include('omeroweb.webredirect.urls')),
    url(r'^(?i)feedback/', include('omeroweb.feedback.urls')),

    url(r'^(?i)api/', include('omeroweb.api.urls')),

    url(r'^index/$', webclient_views.custom_index,
        name="webindex_custom"),
]

urlpatterns += redirect_urlpatterns()


if settings.DEBUG:
    urlpatterns += staticfiles_urlpatterns()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
#
# Copyright (c) 2008 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>, 2008.
#
# Version: 1.0
#

import re

from django.template import Library
from django.templatetags.static import PrefixNode

register = Library()


@register.filter
def wikify(value):
    if value is not None:

        urlier = (r'(http[s]?://|localhost|ftp://|ftps://)'
                  r'(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\)\|,]|'
                  r'(?:%[0-9a-fA-F][0-9a-fA-F]))+')
        wikifier = re.compile(r'\b(%s)\b' % urlier, re.IGNORECASE)
        value = wikifier.sub(r'<a href="\1" target="_blank">\1</a>', value)

        return value

    return value

# happy                :) :-)
# wink                 ;) ;-)
# big smile            :-D :D
# tongue sticking out  :-P :P :-p :p
# surprised / o, no    :-o :O
# sad                  :( :-(
# very sad             :-(( :((
# embarrassed          :">
# wearing sunglasses   B-)
# kiss                 =* :-* :*
# confused             :-/ :-\
# angry                X-( x-( X( x(


@register.filter
def sharewikify(value):
    if value is not None:

        WIKI_WORD = (r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\)\|,]|'
                     '(?:%[0-9a-fA-F][0-9a-fA-F]))+')
        wikifier = re.compile(r'\b(%s)\b' % WIKI_WORD)
        value = wikifier.sub(r'<a href="\1" target="_blank">\1</a>', value)

        # happy :) :-)
        emot1 = re.compile(r'\:[\-]?\)', re.VERBOSE)
        value = emot1.sub(
            r'<img src="%simage/emots/tinymce_smiley-smile18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # wink ;) ;-)
        emot11 = re.compile(r'\;[\-]?\)', re.VERBOSE)
        value = emot11.sub(
            r'<img src="%simage/emots/tinymce_smiley-wink18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # very sad :-(( :((
        emot22 = re.compile(r'\:[\-]?\(\(', re.VERBOSE)
        value = emot22.sub(
            r'<img src="%simage/emots/tinymce_smiley-cry18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # sad :( :-(
        emot2 = re.compile(r'\:[\-]?\(', re.VERBOSE)
        value = emot2.sub(
            r'<img src="%simage/emots/tinymce_smiley-frown18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # kiss =* :-* :*
        emot3 = re.compile(r'[=\:][\-]?\*', re.VERBOSE)
        value = emot3.sub(
            r'<img src="%simage/emots/tinymce_smiley-kiss18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # big smile :-D :D
        emot4 = re.compile(r'\:[\-]?[dD]', re.VERBOSE)
        value = emot4.sub(
            r'<img src="%simage/emots/tinymce_smiley-laughing18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # tongue sticking out :-P :P :-p :p
        emot5 = re.compile(r'\:[\-]?[pP]', re.VERBOSE)
        value = emot5.sub(
            r'<img src="%simage/emots/tinymce_smiley-tongue-out18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # surprised / o, no :-o :O
        emot6 = re.compile(r'\:[\-]?[oO]', re.VERBOSE)
        value = emot6.sub(
            r'<img src="%simage/emots/tinymce_smiley-surprised18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # embarrassed :">
        emot7 = re.compile(r'\:\"\>', re.VERBOSE)
        value = emot7.sub(
            r'<img src="%simage/emots/tinymce_smiley-embarassed18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # wearing sunglasses B-)
        emot8 = re.compile(r'B\-\)', re.VERBOSE)
        value = emot8.sub(
            r'<img src="%simage/emots/tinymce_smiley-cool18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # confused :-/ :-\
        emot9 = re.compile(r'\:\-[\\/]', re.VERBOSE)
        value = emot9.sub(
            r'<img src="%simage/emots/tinymce_smiley-undecided18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # angry X-( x-( X( x(
        emot9 = re.compile(r'[xX][\-]?\(', re.VERBOSE)
        value = emot9.sub(
            r'<img src="%simage/emots/tinymce_smiley-yell18.gif" />'
            % PrefixNode.handle_simple("STATIC_URL"), value)

        # TODO: Beta 4.1
        # quote = re.compile(r'\[quote\](.*)\[/quote\]', re.DOTALL)
        # value = quote.sub(r'<p class="quote">\1</p>', value)

    return value

# happy                :) :-)
# wink                 ;) ;-)
# big smile            :-D :D
# tongue sticking out  :-P :P :-p :p
# surprised / o, no    :-o :O
# sad                  :( :-(
# very sad             :-(( :((
# embarrassed          :">
# wearing sunglasses   B-)
# kiss                 =* :-* :*
# confused             :-/ :-\
# angry                X-( x-( X( x(



from omeroweb.version import omeroweb_version as omero_version
from django.conf import settings


def url_suffix(request):
    suffix = u"?_%s" % omero_version
    return {'url_suffix': suffix}


def base_include_template(request):
    return {'base_include_template': settings.BASE_INCLUDE_TEMPLATE}
